function m_out = m_gui(action, aux, aux2, aux3)
%M_GUI  - MAVIS Control window support

%	branch by action...

switch action,

%-----------------------------------------------------------------------------
%	ABORT:  force exit (debugging)

	case 'ABORT',
		KillGlobals;
		set(0,'ShowHiddenHandles', 'on');
		h = findobj('Tag', 'CONFIG');
		if ~isempty(h), delete(h); end;
		delete(findobj('Tag', 'GUI'));
		delete(findobj('Tag', 'TRAJ'));
		delete(findobj('Tag', 'SPAT'));
		set(0,'ShowHiddenHandles', 'off');
	

%-----------------------------------------------------------------------------
%	CLONE:  clone window for copy, print visibility (aux holds window tag)

	case 'CLONE',
		set(0,'ShowHiddenHandles', 'on');
		h = findobj('Tag', aux);
		set(0,'ShowHiddenHandles', 'off');
		ch = figure('Colormap', get(h, 'Colormap'));
		copyobj(flipud(findobj(h,'type','axes')), ch);
		if strcmp(aux, 'SPAT'), axis square; end;

		
%-----------------------------------------------------------------------------
%	CLOSE:  shutdown

	case 'CLOSE',
		KillGlobals;
		set(0,'ShowHiddenHandles', 'on');
		gui = findobj('Tag', 'GUI');
		set(0,'ShowHiddenHandles', 'off');
		state = get(gui, 'UserData');	
		delete(state.TRAJWIN);
		delete(state.SPATWIN);
		delete(gui);


%-----------------------------------------------------------------------------
%	DELAY:  adjust interstep delay

	case 'DELAY',
		state = get(gcbf, 'UserData');
		if isempty(state.DATA), return; end;
		state.DELAY = get(gcbo, 'Value');
		set(gcbf, 'UserData', state);			

		
%-----------------------------------------------------------------------------
%	GETCFG:  export configuration

	case 'GETCFG',
		[gui, state] = FindGUI;
		if isempty(gui), return; end;
		trajState = get(state.TRAJWIN, 'UserData');
		spatState = get(state.SPATWIN, 'UserData');
		dirs = [strncmp(get(spatState.AXIS,'xdir'),'r',1), strncmp(get(spatState.AXIS,'ydir'),'r',2), strncmp(get(spatState.AXIS,'zdir'),'r',3)]; 
		m_out = struct('CURSOR', state.CURSOR, ...
						'DELAY', state.DELAY, ...
						'HEAD', state.HEAD, ...
						'INTERP', state.INTERP, ...
						'LABEL', state.LABEL, ...
						'TAIL', state.TAIL, ...
						'TIMESTEP', state.TIMESTEP, ...
						'AUTOUPDATE', trajState.AUTOUPDATE, ...
						'BACKSTORE', get(state.TRAJWIN, 'BackingStore'), ...
						'DISPMAP', trajState.DISPMAP, ...
						'FRAME', trajState.FRAME, ...
						'MIDLINES', trajState.MIDLINES, ...
						'PLOTSAMPS', trajState.PLOTSAMPS, ...
						'SHOWTRAJ', trajState.SHOWTRAJ, ...
						'TRAJAUTO', trajState.AUTOSCALE, ...
						'TRAJSPREAD', trajState.SPREAD, ...
						'BLOBSIZE', spatState.BLOBSIZE, ...
						'BLOBTYPE', spatState.BLOBTYPE, ...
						'IMAGE', spatState.IMAGE.DISPLAY, ...
						'LINKMAP', spatState.LINKMAP, ...
						'LINKMODE', spatState.LINKMODE, ...
						'OFFSETS', spatState.OFFSETS, ...
						'PALATE', spatState.PALATE.DISPLAY, ...
						'PREFVIEW', spatState.PREFVIEW, ...
						'SPATAUTO', spatState.AUTOSCALE, ...
						'SPATDIRS', dirs, ...
						'SPATGRID', spatState.GRID, ...
						'SPATMODE', spatState.MODE, ...
						'SPATSPREAD', spatState.SPREAD, ...
						'SPATVIEW', get(spatState.AXIS, 'View'), ...
						'PIXPERMM', spatState.IMAGE.PIXPERMM);
						
		
%-----------------------------------------------------------------------------
%	GETDATA:  export data
%
%	AUX == 0	- return state.DATA as is
%		== 1	- return displayed trajectories only, in DISPMAP ordering
%					within current HEAD:TAIL bounds

	case 'GETDATA',
		global msecsToSamps;
		[gui, state] = FindGUI;
		if isempty(gui), return; end;
		if aux,
			trajState = get(state.TRAJWIN, 'UserData');
			m_out = state.DATA(trajState.DISPMAP);
			for t = 1 : length(m_out),
				head = msecsToSamps(state.HEAD, m_out(t).SRATE);
				tail = msecsToSamps(state.TAIL, m_out(t).SRATE);
				m_out(t).SIGNAL = m_out(t).SIGNAL(head:tail,:);
				if strcmp(m_out(t).SCALING.TYPE, 'COMMON'),		% recompute scaling range
					m_out(t).SCALING.RANGE = [min(m_out(t).SIGNAL)' max(m_out(t).SIGNAL)'];
				end;
				for i = length(m_out(t).LABELS) : -1 : 1,		% reset label offsets; discard those out of range
					if m_out(t).LABELS(i).OFFSET>=state.HEAD & m_out(t).LABELS(i).OFFSET<=state.TAIL,
						m_out(t).LABELS(i).OFFSET = m_out(t).LABELS(i).OFFSET - state.HEAD;
					else,
						m_out(t).LABELS(i) = [];
					end;
				end;
			end;
		else,
			m_out = state.DATA;
		end;
		

%-----------------------------------------------------------------------------
%	GETLAB:  export labels

	case 'GETLAB',
		[gui, state] = FindGUI;
		if isempty(gui), return; end;
		i = 0;
		for t = 1 : length(state.DATA),
			if ~isempty(state.DATA(t).LABELS),
				i = i + 1;
				m_out(i) = struct('NAME', state.DATA(t).NAME, ...
								'LABELS', state.DATA(t).LABELS);
			end;
		end;
		if ~i, m_out = []; end;		% no labels found
		
		
%-----------------------------------------------------------------------------
%	GETVAL:  get trajectory value at offset 
%
%	aux is state, aux2 is trajectory index, aux3 is msec offset
%	returns value using current state.INTERP value

	case 'GETVAL',
		global msecsToSamps sampsToMsecs I_NEAREST I_LINEAR I_CUBIC;
		state = get(gcbf, 'UserData');
		state = aux;
		traj = aux2;
		offset = aux3;
		if offset < 0, offset = 0; end;
		sRate = state.DATA(traj).SRATE;
		sMax = msecsToSamps(state.MAXTAIL, sRate);
		switch state.INTERP,
			case I_NEAREST,		% nearest neighbor
				s = round(offset*sRate/1000)+1;
				if s > sMax; s = sMax; end;
				m_out = state.DATA(traj).SIGNAL(s,:);
			case I_LINEAR,		% linear interpolation
				s = msecsToSamps(offset, sRate);
				if s >= sMax,
					m_out = state.DATA(traj).SIGNAL(sMax,:);
				else,
					v = state.DATA(traj).SIGNAL(s:s+1,:);
					m_out = v(1,:) + (v(2,:)-v(1,:))*(offset-sampsToMsecs(s,sRate))/(1000/sRate);
				end;
			case I_CUBIC,		% cubic interpolation (4 pt neighborhood)
				s1 = msecsToSamps(offset, sRate) - 1;
				s2 = s1 + 3;
				if s1 < 1, s1 = 1; end;
				if s2 > sMax, s2 = sMax; end;
				s = [s1 : s2];
				v = state.DATA(traj).SIGNAL(s,:);
				sx = offset*sRate/1000+1;
				switch size(v,1),
					case 1,
						m_out = v;
					case 2,
						m_out = interp1(s, v, sx, '*linear');
					otherwise,
						m_out = interp1(s, v, sx, '*cubic');
				end;
		end;
		
	
%-----------------------------------------------------------------------------
%	IMPORT:  load data from supported formats

	case 'IMPORT',
		[dataSet, setName] = m_load(aux);
		if ~isempty(dataSet),
			m_gui('LOADSET', dataSet, setName);
		end;
		
	
%-----------------------------------------------------------------------------
%	INIT:  initialize GUI

	case 'INIT',

% check for MAVIS already active
		set(0,'ShowHiddenHandles', 'on');
		gui = findobj('Tag', 'GUI');
		set(0,'ShowHiddenHandles', 'off');
		if isempty(gui),
			m_init('MAVIS');		% not found - initialize
		else,
			state = get(gui, 'UserData');
			figure(state.TRAJWIN);
			figure(state.SPATWIN);
			figure(gui);			% active - pop windows to front
		end;

	
%-----------------------------------------------------------------------------
%	LOADCFG:  load configuration from aux

	case 'LOADCFG',
		global fixstr;
		[gui, state] = FindGUI;
		if isempty(gui), return; end;

		if nargin < 2,		% load interactively
			[file, pathName] = uigetfile('*.mat', 'Select MAVIS cfg file');
			if file == 0, return; end; 	% cancelled
			fileName = [pathName file];
			vars = who('-file', fileName);		% variables in selected file
			if strmatch('cfg', char(vars), 'exact'),
				load(fileName, 'cfg');			% load the one called CFG
			else,
				set(warndlg(['File ', file, ' does not contain a variable called cfg'], ''), 'WindowStyle', 'modal');
				return;
			end;
		else,
			cfg = aux;
		end;
		
		trajState = get(state.TRAJWIN, 'UserData');
		spatState = get(state.SPATWIN, 'UserData');
		cfgFields = char(fieldnames(cfg));
		
		state.DELAY = cfg.DELAY;
		state.INTERP = cfg.INTERP;
		state.LABEL = cfg.LABEL;
		state.TIMESTEP = cfg.TIMESTEP;
		
		trajState.AUTOUPDATE = cfg.AUTOUPDATE;
		set(state.TRAJWIN, 'BackingStore', cfg.BACKSTORE);
		trajState.DISPMAP = cfg.DISPMAP;
		trajState.FRAME = cfg.FRAME;
		trajState.MIDLINES = cfg.MIDLINES;
		trajState.PLOTSAMPS = cfg.PLOTSAMPS;
		trajState.AUTOSCALE = cfg.TRAJAUTO;
		trajState.SPREAD = cfg.TRAJSPREAD;
		if ~isempty(strmatch('SHOWTRAJ', cfgFields, 'exact')), 
			trajState.SHOWTRAJ = cfg.SHOWTRAJ;
		else,
			trajState.SHOWTRAJ = [1 1 1 1];
		end;
		
		spatState.BLOBSIZE = cfg.BLOBSIZE;
		spatState.BLOBTYPE = cfg.BLOBTYPE;
		spatState.IMAGE.DISPLAY = cfg.IMAGE;
		spatState.LINKMAP = cfg.LINKMAP;
		spatState.LINKMODE = cfg.LINKMODE;
		spatState.OFFSETS = cfg.OFFSETS;
		spatState.PALATE.DISPLAY = cfg.PALATE;
		spatState.AUTOSCALE = cfg.SPATAUTO;
		spatState.GRID = cfg.SPATGRID;
		spatState.MODE = cfg.SPATMODE;
		spatState.SPREAD = cfg.SPATSPREAD;
		spatState.IMAGE.PIXPERMM = cfg.PIXPERMM;
		ds = ['normal';'revers'];
		di = cfg.SPATDIRS+1;
		set(spatState.AXIS, ...
			'View', cfg.SPATVIEW, ...
			'XDir', ds(di(1)), ...
			'YDir', ds(di(2)), ...
			'ZDir', ds(di(3)));
		if ~isempty(strmatch('PREFVIEW', cfgFields, 'exact')), 
			spatState.PREFVIEW = cfg.PREFVIEW;
		else,
			spatState.PREFVIEW = [];
		end;
		
		if trajState.AUTOUPDATE, s = 'on'; else, s = 'off'; end;
		set(state.MENUS(:,1), 'Checked', s);
		if cfg.BACKSTORE,  s = 'on'; else, s = 'off'; end;
		set(state.MENUS(:,2), 'Checked', s);
		if trajState.PLOTSAMPS, s = 'on'; else, s = 'off'; end;
		set(state.MENUS(:,3), 'Checked', s);
		if trajState.MIDLINES, s = 'on'; else, s = 'off'; end;
		set(state.MENUS(:,4), 'Checked', s);
		if state.LABEL.DISPLAY, s = 'on'; else, s = 'off'; end;
		set(state.MENUS(:,6), 'Checked', s);

		set(gui, 'UserData', state);
		set(state.TRAJWIN, 'UserData', trajState);
		set(state.SPATWIN, 'UserData', spatState);
		m_traj('INIT', state);
		m_spat('INIT', state);
		
		state = get(gui, 'UserData');
		state.CURSOR = cfg.CURSOR;
		if state.CURSOR > state.MAXTAIL, state.CURSOR = state.MAXTAIL; end;
		state.HEAD = cfg.HEAD;
		if state.HEAD >= state.MAXTAIL - state.MINSTEP, state.HEAD = state.MAXTAIL - state.MINSTEP; end;
		state.TAIL = cfg.TAIL;
		if state.TAIL <= state.HEAD + state.MINSTEP, 
			state.TAIL = state.HEAD + state.MINSTEP; 
		elseif state.TAIL > state.MAXTAIL, 
			state.TAIL = state.MAXTAIL; 
		end;
		set(state.HEADF, 'String', fixstr(state.HEAD));
		set(state.TAILF, 'String', fixstr(state.TAIL));
		set(state.CURSF, 'String', fixstr(state.CURSOR));
		set(gui, 'UserData', state);
		m_traj('UPDATE', get(state.TRAJWIN, 'UserData'));
		m_traj('DRAWCURS', state);
		m_spat('UPDATE', state);
		figure(gui);		
		
		
%-----------------------------------------------------------------------------
%	LOADIMG:  load image

 	case 'LOADIMG',
		[gui, state] = FindGUI;
		if isempty(gui), return; end;

% load interactively
		if nargin < 3,
			switch aux,
				case 'MAT',		% load from MAT file
					[file, pathName] = uigetfile('*.mat', 'Select Image MAT file');
					if file == 0, return; end; 	% cancelled
					fileName = [pathName file];
					vars = who('-file', fileName);		% variables in selected file
					if strmatch('img', char(vars), 'exact'),
						load(fileName, 'img');			% load the one called IMG
						imgName = file;
						i = findstr(imgName, '.');
						if ~isempty(i), imgName = imgName(1:i(1)-1); end;
					else,
						set(warndlg(['File ', file, ' does not contain a variable called img'], ''), 'WindowStyle', 'modal');
						return;
					end;
				case 'SCAN',	% load from Microbeam scan
					[file, pathName] = uigetfile('I*', 'Select Microbeam scan');
					if file == 0, return; end; 	% cancelled
					fileName = [pathName file];
					img = LoadMBeamScan(fileName);
					imgName = file;
				case 'IMG',		% load from image file
					[file, pathName] = uigetfile('*', 'Select Image file');
					if file == 0, return; end; 	% cancelled
					fileName = [pathName file];
					eval(['fInfo = imfinfo(''',fileName,''');'], ['fInfo = [];']);
					if isempty(fInfo),
						set(warndlg(['File ', file, ' is not a supported image format'], ''), 'WindowStyle', 'modal');
						return;
					end;
					if strcmp(fInfo.Format, 'tif'), s = warning; warning off; end;
					img = imread(fileName, fInfo.Format);
					if strcmp(fInfo.Format, 'tif'), warning(s); end;
					imgName = file;
					i = findstr(imgName, '.');
					if ~isempty(i), imgName = imgName(1:i(1)-1); end;
			end;
				
% load from specified aux args
		else,
			img = aux;
			imgName = aux2;
		end;

% average across color channels if necessary
		if ndims(img) > 2,
			img = mean(double(img),3);
		end;
		
		spatState = get(state.SPATWIN, 'UserData');
		spatState.IMAGE.DATA = img;
		spatState.IMAGE.NAME = imgName;
		spatState.IMAGE.DISPLAY = 1;
		set(state.SPATWIN, 'UserData', spatState);
		m_spat('INIT', state); 
		figure(spatState.GUI);

		
%-----------------------------------------------------------------------------
%	LOADLBL:  load (melba) label data

	case 'LOADLBL',
		[gui, state] = FindGUI;
		if isempty(gui), return; end;
		labels = aux;
		if ~HasFields(labels, {'NAME','OFFSET','HOOK'}), return; end;
		set(state.TRAJWIN, 'HandleVisibility', 'on');
		for i = 1 : length(labels),
			label = struct('NAME', labels(i).NAME, ...
							'OFFSET', labels(i).OFFSET, ...
							'NOTE', labels(i).HOOK, ...
							'VALUE', []);
			m_label('MAKE', label, state);
			state = get(state.GUI, 'UserData');
		end;
		set(state.TRAJWIN, 'HandleVisibility', 'callback');


%-----------------------------------------------------------------------------
%	LOADPAL:  load palate trace data

 	case 'LOADPAL',
		[gui, state] = FindGUI;
		if isempty(gui), return; end;

		if nargin < 2,		% load interactively
			[file, pathName] = uigetfile('*.mat', 'Select Palate Trace datafile');
			if file == 0, return; end; 	% cancelled
			fileName = [pathName file];
			vars = who('-file', fileName);		% variables in selected file
			if strmatch('pal', char(vars), 'exact'),
				load(fileName, 'pal');			% load the one called PAL
				palName = file;
				i = findstr(palName, '.');
				if ~isempty(i), palName = palName(1:i(1)-1); end;
			else,
				set(warndlg(['File ', file, ' does not contain a variable called pal'], ''), 'WindowStyle', 'modal');
				return;
			end;
		else,
			pal = aux;
			palName = aux2;
		end;

		spatState = get(state.SPATWIN, 'UserData');
		spatState.PALATE.DATA = pal;
		spatState.PALATE.NAME = palName;
		spatState.PALATE.DISPLAY = 1;
		set(state.SPATWIN, 'UserData', spatState);
		m_spat('INIT', state); 
		figure(spatState.GUI);
		
		
%-----------------------------------------------------------------------------
%	LOADSET:  load MAVIS dataset

 	case 'LOADSET',
		switch nargin,
			case 1, 
				aux = [];
				aux2 = [];
			case 2,			% make workspace variable accessible
				s = warning; warning off; eval(['global ', inputname(2)]); warning(s);
				aux2 = inputname(2);
			case 3,
		end;
		if nargin < 2, aux = []; end;

		[dataSet, setName, cfg, pal, img, pha] = m_load('NATIVE', aux, aux2);
		if isempty(dataSet), return; end;	% interactive load cancelled

		set(0,'ShowHiddenHandles', 'on');
		gui = findobj('Tag', 'GUI');
		set(0,'ShowHiddenHandles', 'off');
		state = get(gui, 'UserData');		% set the loaded name field
		set(state.NAMEF, 'String', setName);

		set(state.TRAJWIN, 'Name', ['TRAJECTORY:  ', setName]);
		state.DATA = dataSet;
		set(gui, 'UserData', state);
		trajState = get(state.TRAJWIN, 'UserData');
		trajState.FRAME = 1;			% 1st trajectory is initial framing data
		trajState.DISPMAP = [1 : length(dataSet)];	% display all trajectories initially
		set(state.TRAJWIN, 'UserData', trajState);
		spatState = get(state.SPATWIN, 'UserData');
		spatState.LINKMAP = [];			% kill the link map

		if ~(isempty(pal) & isempty(img)),
			if ~isempty(pal), 
				spatState.PALATE.DATA = pal; 
				spatState.PALATE.NAME = 'pal';
				spatState.PALATE.PHA = pha;
			end;			
			if ~isempty(img), spatState.IMAGE.DATA = img; spatState.IMAGE.NAME = 'img'; end;
		end;
		set(state.SPATWIN, 'UserData', spatState);
		
		m_traj('INIT', state);
		m_spat('INIT', state);
		if isempty(cfg),
			m_spat('RESET', get(state.SPATWIN, 'UserData'));
			figure(gui);
		else,
			m_gui('LOADCFG', cfg);
		end;
		
	
%-----------------------------------------------------------------------------
%	MOTION:  motion handlers

	case 'MOTION',
		[gui, state] = FindGUI;
		if isempty(gui), return; end;
		switch(aux),
			case 'HEAD',
				UpdatePos(state.HEAD, gui, state);
			case 'RREV',
				if state.MOTION,
					if state.MOTION < 0,
						btnup(gui, 'moveButts', 'RREV');
						state.MOTION = 0;
						set(gui, 'UserData', state);
					else,
						btnup(gui, 'moveButts', 'FFWD');
						btndown(gui, 'moveButts', 'RREV');
						MotionLoop(-1, gui);
					end;
				else,
					btndown(gui, 'moveButts', 'RREV');
					MotionLoop(-1, gui);
				end;
			case 'REV ',
				newPos = state.CURSOR - state.TIMESTEP;
				if newPos < state.HEAD, newPos = state.HEAD; end;
				UpdatePos(newPos, gui, state);
			case 'STOP',
				if state.MOTION < 0,
					btnup(gui, 'moveButts', 'RREV');
				else,
					btnup(gui, 'moveButts', 'FFWD');
				end;
				btnstate('set', gui, 'moveButts', zeros(1,7));
				state.MOTION = 0;
				set(gui, 'UserData', state);
			case 'FWD ',
				newPos = state.CURSOR + state.TIMESTEP;
				if newPos > state.TAIL, newPos = state.TAIL; end;
				UpdatePos(newPos, gui, state);
			case 'FFWD',
				if state.MOTION,
					if state.MOTION > 0,
						btnup(gui, 'moveButts', 'FFWD');
						state.MOTION = 0;
						set(gui, 'UserData', state);
					else,
						btnup(gui, 'moveButts', 'RREV');
						btndown(gui, 'moveButts', 'FFWD');
						MotionLoop(1, gui);
					end;
				else,
					btndown(gui, 'moveButts', 'FFWD');
					MotionLoop(1, gui);
				end;
			case 'TAIL',
				UpdatePos(state.TAIL, gui, state);
		end;
		
	
%-----------------------------------------------------------------------------
%	SAVECFG:  save configuration

	case 'SAVECFG',
		[gui, state] = FindGUI;
		if isempty(gui), return; end;
		
		[file, pathName] = uiputfile('config.mat', 'Save configuration as');
		if file == 0, return; end;		% cancelled
		cfg = m_gui('GETCFG');
		save([pathName, file], 'cfg');

		
%-----------------------------------------------------------------------------
%	SAVELABS:  save labels to tab-delimited file
%
% output has msecs OFFSETS as rows, trajectory X,Y as columns

	case 'SAVELABS',
		[gui, state] = FindGUI;
		if isempty(gui), return; end;

		[fName, pathName] = uiputfile([get(state.NAMEF,'string'),'.lab'], 'Save labels as');
		if fName == 0, return; end;		% cancelled
		fName = [pathName, fName];
		
% write headers
		[fp,msg] = fopen(fName, 'wt');
		if fp == -1,
			error(['Error attempting to open ' fName]);
			return;
		end;
		fprintf(fp, 'OFFSET');
		offsets = [];
		for t = 1 : length(state.DATA),
			if size(state.DATA(t).SIGNAL,2) > 1,
				fprintf(fp, '\t%s_X\t%s_Y\t%s_T', state.DATA(t).NAME, state.DATA(t).NAME, state.DATA(t).NAME);
				
% build a single list of all label offsets
				for n = 1 : length(state.DATA(t).LABELS),
					lo = state.DATA(t).LABELS(n).OFFSET;
					if isempty(offsets), offsets = lo; end;
					for i = 1 : length(offsets),
						if offsets(i) == lo, break; end;
						if offsets(i) > lo,
							if i == 1,
								offsets = [lo offsets];
							else,
								offsets = [offsets(1:i-1) lo offsets(i:end)];
							end;
							break;
						end;
					end;
					if offsets(end) < lo, offsets(end+1) = lo; end;
				end;
				
			end;
		end;
		fprintf(fp, '\n');

% write trajectory values at label offsets
		for i = 1 : length(offsets),
			fprintf(fp, '%g', offsets(i));
			for t = 1 : length(state.DATA),
				if size(state.DATA(t).SIGNAL,2) > 1,
					xyt = [];
					for n = 1 : length(state.DATA(t).LABELS),
						if state.DATA(t).LABELS(n).OFFSET == offsets(i),
							xyt = state.DATA(t).LABELS(n).VALUE;
							break;
						end;
					end;
					if isempty(xyt),
						fprintf(fp, '\t\t\t');
					else,
						fprintf(fp, '\t%g\t%g\t%g', xyt);
					end;
				end;
			end;
			fprintf(fp, '\n');
		end;

		fclose(fp);
		fprintf('Labels written to %s.\n', fName);
		
%-----------------------------------------------------------------------------
%	SAVESEL:  save selection to workspace

	case 'SAVESEL',
		global msecsToSamps;
		[gui, state] = FindGUI;
		if isempty(gui), return; end;
		name = GetName([get(state.NAMEF,'string'),'_sel']);
		if ~isempty(name),
			trajState = get(state.TRAJWIN, 'UserData');
			data = state.DATA(trajState.DISPMAP);
			for t = 1 : length(data),
				head = msecsToSamps(state.HEAD, data(t).SRATE);
				tail = msecsToSamps(state.TAIL, data(t).SRATE);
				data(t).SIGNAL = data(t).SIGNAL(head:tail,:);
				if strcmp(data(t).SCALING.TYPE, 'COMMON'),		% recompute scaling range
					data(t).SCALING.RANGE = [min(data(t).SIGNAL)' max(data(t).SIGNAL)'];
				end;
				for i = length(data(t).LABELS) : -1 : 1,		% reset label offsets; discard those out of range
					if data(t).LABELS(i).OFFSET>=state.HEAD & data(t).LABELS(i).OFFSET<=state.TAIL,
						data(t).LABELS(i).OFFSET = data(t).LABELS(i).OFFSET - state.HEAD;
					else,
						data(t).LABELS(i) = [];
					end;
				end;
			end;
			assignin('base', name, data);
			evalin('base', name)		% for listing
		end;

		
%-----------------------------------------------------------------------------
%	SAVESET:  save data & configuration to MAT file

	case 'SAVESET',
		[gui, state] = FindGUI;
		if isempty(gui), return; end;
		
		setName = get(state.NAMEF, 'String');
		[file, pathName] = uiputfile([setName '.mat'], 'Save dataset as');
		if file == 0, return; end;		% cancelled
		setName = file;
		i = findstr(setName, '.');
		if ~isempty(i), setName = setName(1:i(1)-1); end;
		eval([setName, '= state.DATA;']);
		cfg = m_gui('GETCFG');
		varList = ['''' [pathName, file] ''',''' setName ''',''cfg'''];
		spatState = get(state.SPATWIN, 'UserData');
		if ~isempty(spatState.PALATE.DATA),
			pal = spatState.PALATE.DATA;
			varList = [varList ',''pal'''];
		end;
		if ~isempty(spatState.IMAGE.DATA),
			img = spatState.IMAGE.DATA;
			varList = [varList ',''img'''];
		end;
		eval(['save(',varList,');']);
		
		
%-----------------------------------------------------------------------------
%	SET:  load value from field

	case 'SET',
		state = get(gcbf, 'UserData');
		if isempty(state.DATA), return; end;
		global fixstr;
		v = str2num(get(gcbo, 'String'));
		if isempty(v), v = 0; end;
		if v < 0, v = 0; elseif v > state.MAXTAIL, v = state.MAXTAIL; end;
		switch aux,
			case 'CURSOR', 	
				state.CURSOR = v;
			case 'HEAD',	
				if v >= state.TAIL-state.MINSTEP, v = state.TAIL - state.MINSTEP; end;
				state.HEAD = v;
			case 'TAIL',
				if v <= state.HEAD+state.MINSTEP, v = state.HEAD + state.MINSTEP; end;
				if v > state.MAXTAIL, v = state.MAXTAIL; end;
				state.TAIL = v;
		end;
		set(gcbo, 'String', fixstr(v));	
		set(gcbf, 'UserData', state);
		m_traj('UPDATE', get(state.TRAJWIN, 'UserData'));
		if strcmp(aux, 'CURSOR'), m_spat('UPDATE', state); end;
		

%-----------------------------------------------------------------------------
%	TIMESTEP:  change timestep (aux = 1 by slider; 2 by field)

	case 'TIMESTEP',
		state = get(gcbf, 'UserData');
		switch aux,
			case 1,			% slider
				v = get(gcbo, 'Value');
				set(get(gcbo, 'UserData'), 'String', num2str(v));
			case 2,			% field
				v = str2num(get(gcbo, 'String'));
				vv = v;
				if isempty(v) | v < 1,
					v = 1;
				elseif v > 500,		% max value (msecs)
					v = 500;
				end;
				if v ~= vv, set(gcbo, 'String', num2str(v)); end;
				set(get(gcbo, 'UserData'), 'Value', v);
		end;
		state.TIMESTEP = v;
		set(gcbf, 'UserData', state);
		

%-----------------------------------------------------------------------------
%	WINDOW:  pop specified window

 	case 'WINDOW',
		if nargin > 1,		% pop window
			set(0,'ShowHiddenHandles', 'on');
			h = findobj('Tag', aux);
			set(0,'ShowHiddenHandles', 'off');
			figure(h);
			return;
		end;

% set default positions
		set(0,'ShowHiddenHandles', 'on');
		gui = findobj('Tag', 'GUI');
		set(0,'ShowHiddenHandles', 'off');
		state = get(gui, 'UserData');
		trajState = get(state.TRAJWIN, 'UserData');
		spatState = get(state.SPATWIN, 'UserData');
		figure(state.TRAJWIN);
		set(state.TRAJWIN, 'Position', trajState.DEFPOS);
		figure(state.SPATWIN);
		set(state.SPATWIN, 'Position', spatState.DEFPOS);
		figure(gui);
		set(gui, 'Position', state.DEFPOS);
		
		
%-----------------------------------------------------------------------------
%	error

	otherwise,
		error(['M_GUI:  unrecognized action (', action, ')']);
	
end;


%=============================================================================
%	DELAY  - pause for specied delay (secs)

function Delay(dt)

if ~dt, return; end;
tic;
while toc < dt, end;


%=============================================================================
%	FINDGUI  - locate GUI object; returns [] if state data not initialized

function [gui, state] = FindGUI

set(0,'ShowHiddenHandles', 'on');
gui = findobj('Tag', 'GUI');
set(0,'ShowHiddenHandles', 'off');
if isempty(gui), state = []; return; end;
state = get(gui, 'UserData');
if isempty(state.DATA), gui = []; end;
		


%=============================================================================
% GETNAME  - get export variable name

function name = GetName(name)

figPos = get(0, 'ScreenSize');
width = 200;
height = 100;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];

cfg = dialog('Name', 'Save selection in workspace as...', ...
	'Position', figPos, ...
	'ButtonDownFcn', '', ...
	'KeyPressFcn', 'set(gcbf,''UserData'',1);uiresume', ...
	'UserData', 0);

% name field
eh = uicontrol(cfg, ...
	'Position', [20 60 width-40 20], ...
	'Style', 'edit', ...
	'HorizontalAlignment', 'left', ...
	'Callback', 'set(gcbf,''UserData'',1);uiresume', ...
	'String', name);

% OK, cancel buttons
uicontrol(cfg, ...		% buttons
	'Position',[width/2-70 15 60 25], ...
	'String','OK', ...
	'Callback','set(gcbf,''UserData'',1);uiresume');
uicontrol(cfg, ...
	'Position',[width/2+10 15 60 25], ...
	'String','Cancel', ...
	'Callback','uiresume');

% wait for input
uiwait(cfg);
if get(cfg, 'UserData'),
	name = get(eh, 'string');
else,
	name = [];
end;
delete(cfg);


%=============================================================================
%	KILLGLOBALS  - delete MAVIS globals

function KillGlobals

clear global msecsToSamps sampsToMsecs fixstr;
clear global I_NEAREST I_LINEAR I_CUBIC;
clear global SLOP M_NONE M_HEAD M_TAIL M_CURS M_LABEL;
clear global D_BLOBS D_NOERASE D_VELOCITY D_GHOSTS HISTLEN;
clear global L_OFF L_OPEN L_CLOSE L_NSPLINE;


%=============================================================================
%	MOTIONLOOP  - step position while motion active

function MotionLoop(dir, gui)

state = get(gui, 'UserData');
state.MOTION = dir;
set(gui, 'UserData', state);

while dir,
	state = get(gui, 'UserData');		% refresh state to detect interrupt
	dir = state.MOTION;
	newPos = state.CURSOR + state.TIMESTEP*dir;
	if newPos < state.HEAD,				% wrap around
		newPos = state.TAIL;
	elseif newPos > state.TAIL,
		newPos = state.HEAD;
	end;
	UpdatePos(newPos, gui, state);
	Delay(state.DELAY);
	drawnow;
end;


%=============================================================================
%	UPDATEPOS  - update cursor position

function UpdatePos(newPos, gui, state)

global fixstr;
state.CURSOR = newPos;			% new cursor offset (msecs)
set(gui, 'UserData', state);
set(state.CURSF, 'String', fixstr(newPos));
m_traj('DRAWCURS', state);
m_spat('UPDATE', state);
