function m_init(titleString)
%M_INIT  - MAVIS initialization

% 	init constants

global msecsToSamps sampsToMsecs fixstr;
global I_NEAREST I_LINEAR I_CUBIC;
global SLOP M_NONE M_HEAD M_TAIL M_CURS M_LABEL;
global D_BLOBS D_NOERASE D_VELOCITY D_GHOSTS HISTLEN;
global L_OFF L_OPEN L_CLOSE L_NSPLINE;
		
% sample <-> time conversion
msecsToSamps = inline('floor(msecs*sRate/1000)+1','msecs','sRate');
sampsToMsecs = inline('1000*(samps-1)/sRate','samps','sRate');

% allow one decimal digit
fixstr = inline('num2str(round(x*10)/10)');	

% interpolation modes
I_NEAREST = 1;
I_LINEAR = 2;
I_CUBIC = 3;

% movement modes		
M_NONE	= 0;				% movement modes...
M_HEAD	= 1;
M_TAIL	= 2;
M_CURS	= 3;
M_LABEL = 4;

SLOP	= 10;				% grabbing slop

% display modes
D_BLOBS   = 1;					
D_NOERASE = 2;
D_VELOCITY= 3;
D_GHOSTS  = 4;
		
HISTLEN	= 10;				% history buffer length (number of ghosts)

% linking modes
L_OFF     = 1;							
L_OPEN    = 2;
L_CLOSE   = 3;
L_NSPLINE = 4;

%---- GUI ----

% init internal state

state = struct('GUI', [], ...		% control window
			'TRAJWIN', [], ...		% trajectory window
			'SPATWIN', [], ...		% spatial display window
			'NAMEF', [], ...		% loaded dataset name field
			'CURSF', [], ...		% cursor field
			'HEADF', [], ...		% head field
			'TAILF', [], ...		% tail field
			'XVALF', [], ...		% X val field
			'YVALF', [], ...		% Y val field
			'ZVALF', [], ...		% Z val field
			'CURSOR', [], ...		% cursor loc (msecs)
			'HEAD', [], ...			% selection bounds (msecs)
			'TAIL', [], ...
			'TIMESTEP', 20, ...		% scrolling timestep (msecs)
			'INTERP', I_LINEAR, ...	% interpolation mode
			'MOTION', 0, ...		% motion flag (-1 rev, 0 stopped, +1 fwd)
			'DELAY', 0, ...			% inter-update delay (secs)
			'MAXTAIL', [], ...		% length of shortest trajectory (msecs)
			'MINSTEP', [], ...		% min head:tail separation (msecs)
			'MENUS', [], ...		% checked menu handles
			'LABEL', struct('DISPLAY', 1, ...	% display labels by default
						'ANNOTATE', 1, ...		% annotate by default
						'ALLTRAJ', 0, ...		% apply to clicked trajectory only
						'PROC', [], ...			% use default labelling procedure
						'USERDATA', []), ...	% user proc data
			'DATA', []);			% loaded data
	
% 	set up GUI window

width = 230; height = 275;
screenSize = get(0, 'ScreenSize');
if strcmp(computer, 'MAC2'), screenSize(4) = screenSize(4)+40; end;
if screenSize(4)-3*width-160 < 10, w2 = width; else w2 = 2*width; end;
figPos = [screenSize(3)-width-25, screenSize(4)-w2-width-195, width, height];
if w2 > width, figPos(1) = figPos(1) - .5*width; end;
if figPos(2) < 15, figPos(2) = 15; end;
if strncmp(computer, 'SGI', 3), as = 'left'; else, as = 'right'; end;	% SGI bug:  doesn't like right alignment

gui = dialog('Name', titleString, ...
	'Position', figPos, ...
	'WindowStyle', 'normal', ...
	'ButtonDownFcn', '', ...
	'Tag', 'GUI', ...
	'MenuBar', 'none', ...
	'CloseRequestFcn', 'm_gui CLOSE;');

state.GUI = gui;
state.DEFPOS = figPos;


%	squawk button

uicontrol(gui, ...
	'Style', 'push', ...
	'Position', [20 235 40 25], ...
	'String', 'Play', ...
	'toolTipString', 'Play Selection', ...
	'CallBack', 'm_traj YACK;');
	

% 	dataset selector

pb = uicontrol(gui, ...
	'Style', 'push', ...
	'Position', [65 235 40 25], ...
	'String', 'Data', ...
	'toolTipString', 'Open Data Set', ...
	'CallBack', 'm_gui LOADSET;');
state.NAMEF = uicontrol(gui, ...
	'Style', 'edit', ...
	'HorizontalAlignment', 'left', ...
	'Position', [110 237 105 20], ...
	'Callback', 'eval([''m_gui(''''LOADSET'''', '', get(gcbo, ''String''), '')'']);');

% 	movement buttons

buttonIDs = [ 'HEAD';'RREV';'REV ';'STOP';'FWD ';'FFWD';'TAIL' ]; 
iconFuns = [
	'text(.5,.5,''|<'',''Horiz'',''center'',''Color'',[0 0 0])'
	'text(.5,.5,''<<'',''Horiz'',''center'',''Color'',[0 0 0])'
	'text(.5,.5,''<'',''Horiz'',''center'',''Color'',[0 0 0]) '
	'text(.5,.5,''0'',''Horiz'',''center'',''Color'',[0 0 0]) '
	'text(.5,.5,''>'',''Horiz'',''center'',''Color'',[0 0 0]) '
	'text(.5,.5,''>>'',''Horiz'',''center'',''Color'',[0 0 0])'
	'text(.5,.5,''>|'',''Horiz'',''center'',''Color'',[0 0 0])' ];
pressTypes = [ 'flash ';'toggle';'flash ';'flash ';'flash ';'toggle';'flash ' ];
callbacks = [
	'm_gui(''MOTION'', ''HEAD'');'
	'm_gui(''MOTION'', ''RREV'');'
	'm_gui(''MOTION'', ''REV '');'
	'm_gui(''MOTION'', ''STOP'');'
	'm_gui(''MOTION'', ''FWD '');'
	'm_gui(''MOTION'', ''FFWD'');'
	'm_gui(''MOTION'', ''TAIL'');' ];
	
btngroup(gui, 'GroupID', 'moveButts', ...
	'ButtonID', buttonIDs, ...
	'PressType', pressTypes, ...
	'Position', [20 195 193 25], ...
	'Units', 'pixels', ...
	'GroupSize', [1 7], ...
	'Orientation', 'horizontal', ...
	'BevelWidth', .1, ...
	'IconFunctions', iconFuns, ...
	'Callbacks', callbacks);
			

% timestep

uicontrol(gui, ...		
	'Style', 'text', ...
	'HorizontalAlignment', as, ...
	'String', 'Step', ...
	'Position', [9 159 37 17]);
ss = uicontrol(gui, ...
	'Position',[50 159 100 20], ...
	'Style','slider', ...
	'Value', state.TIMESTEP, ...
	'Min', 1, 'Max', 500, ...
	'Callback', 'm_gui(''TIMESTEP'', 1);');
sf = uicontrol(gui, ...
	'Position',[154 159 59 20], ...
	'Style','edit',...
	'String', num2str(state.TIMESTEP), ...
	'HorizontalAlignment', 'left', ...
	'Userdata', ss, ...
	'Callback', 'm_gui(''TIMESTEP'', 2);');
set(ss, 'Userdata', sf);


% 	delay

uicontrol(gui, ...
	'Style', 'text', ...
	'HorizontalAlignment', as, ...
	'String', 'Delay', ...
	'Position', [7 125 39 17]);
slider = uicontrol(gui, ...
	'Style', 'slider', ...
	'Position', [50 125 161 20], ...
	'Value', 0, ...
	'Min', 0, 'Max', 2, ...
	'CallBack', 'm_gui DELAY;');
	
	
% 	cursor

uicontrol(gui, ...
	'Style', 'text', ...
	'HorizontalAlignment', as, ...
	'String', 'Cursor', ...
	'Position', [5 89 43 17]);
state.CURSF = uicontrol(gui, ...
	'Style', 'edit', ...
	'HorizontalAlignment', 'left', ...
	'Position', [50 89 59 20], ...
	'CallBack', 'm_gui(''SET'', ''CURSOR'');');

	
% 	head

uicontrol(gui, ...
	'Style', 'text', ...
	'HorizontalAlignment', as, ...
	'String', 'Head', ...
	'Position', [9 52 38 17]);
state.HEADF = uicontrol(gui, ...
	'Style', 'edit', ...
	'HorizontalAlignment', 'left', ...
	'Position', [50 52 59 20], ...
	'CallBack', 'm_gui(''SET'', ''HEAD'');');

	
% 	tail

uicontrol(gui, ...
	'Style', 'text', ...
	'HorizontalAlignment', as, ...
	'String', 'Tail', ...
	'Position', [12 15 35 17]);
state.TAILF = uicontrol(gui, ...
	'Style', 'edit', ...
	'HorizontalAlignment', 'left', ...
	'Position', [50 15 59 20], ...
	'CallBack', 'm_gui(''SET'', ''TAIL'');');

	
% 	X val

uicontrol(gui, ...
	'Style', 'text', ...
	'HorizontalAlignment', as, ...
	'String', 'X val', ...
	'Position', [113 89 39 17]);
state.XVALF = uicontrol(gui, ...
	'Style', 'edit', ...
	'Enable', 'inactive', ...
	'HorizontalAlignment', 'left', ...
	'Position', [154 89 59 20]);

	
% 	Y val

uicontrol(gui, ...
	'Style', 'text', ...
	'HorizontalAlignment', as, ...
	'String', 'Y val', ...
	'Position', [113 52 39 17]);
state.YVALF = uicontrol(gui, ...
	'Style', 'edit', ...
	'Enable', 'inactive', ...
	'HorizontalAlignment', 'left', ...
	'Position', [154 52 59 20]);

	
% 	Z val

uicontrol(gui, ...
	'Style', 'text', ...
	'HorizontalAlignment', as, ...
	'String', 'Z val', ...
	'Position', [113 15 39 17]);
state.ZVALF = uicontrol(gui, ...
	'Style', 'edit', ...
	'Enable', 'inactive', ...
	'HorizontalAlignment', 'left', ...
	'Position', [154 15 59 20]);

	
%---- trajectory ----

%	colormap planning
if get(0,'ScreenDepth') < 16, 
	numCols = 16;
	numGrays = 32; 
else,
	numCols = 64;
	numGrays = 64; 
end;

%	trajState userdata

trajState = struct('GUI', gui, ...		% GUI window handle
			'AUTOSCALE', 1, ...			% auto scaling enabled
			'AUTOUPDATE', 1, ...		% auto update on by default
			'PLOTSAMPS', 0, ...			% don't plot samples by default
			'MIDLINES', 1, ...			% plot midlines by default
			'FRAME', [], ...			% framing trajectory index (into state.DATA)
			'DISPMAP', [], ...			% displayed trajectory map (indices into state.DATA)
			'SHOWTRAJ', [1 1 1 1], ...	% show all trajectories by default
			'INITIALIZED', 0);			% init req'd

%	set up trajectory window

figPos = [10, 10, screenSize(3)-w2-70, screenSize(4)-70];
if strcmp(computer, 'PCWIN'), figPos(2) = 40; figPos(4) = figPos(4) - 30; end;
state.TRAJWIN = colordef('new', 'black');
set(state.TRAJWIN, ...
	'Name', 'TRAJECTORY', ...
	'NumberTitle', 'off', ...
	'BackingStore', 'on', ...
	'WindowButtonDownFcn', 'm_traj DOWN;', ...
	'Position', figPos, ...
	'CloseRequestFcn', 'm_traj CLOSE;', ...
	'Tag', 'TRAJ', ...
	'HandleVisibility', 'callback', ...
	'MenuBar', 'none');
trajState.DEFPOS = figPos;
set(state.TRAJWIN, ...
	'Colormap', hsv(numCols), ...
	'Visible', 'on', ...
	'UserData', trajState);
	

%---- spatial ----

%	spatState userdata

if strcmp(computer,'PCWIN'), bs = 5; bt = 'diamond'; else, bs = 15; bt = '.'; end;
spatState = struct('GUI', gui, ...		% GUI window handle
			'AUTOSCALE', 1, ...			% auto scaling enabled
			'BLOBTYPE', bt, ...			% default blob type 
			'BLOBSIZE', bs, ...			% default blob size (points)
			'MODE', D_BLOBS, ...		% default display mode
			'GRID', 1, ...				% display grid by default
			'PALATE', struct('DISPLAY', 1, ...	% palate trace
									'NAME', '', ...
									'DATA', [], ...
									'PHA', []), ...
			'IMAGE', struct('DISPLAY', 0, ...	% image
									'PIXPERMM', 1, ...
									'NAME', '', ...
									'DATA', []), ...
			'PLOTMAP', [], ...			% indices of plotted trajectories (into state.DATA)
			'LINKMODE', L_OFF, ...		% default link mode
			'LINKMAP', [], ...			% indices of linked trajectories (into state.DATA)
			'PREFVIEW', [], ...			% no preferred view
			'INITIALIZED', 0);			% init req'd
			
%	set up spatial window
			
figPos = [screenSize(3)-w2-25, screenSize(4)-w2-60, w2, w2];
state.SPATWIN = colordef('new', 'black');
set(state.SPATWIN, ...
	'Name', 'SPATIAL', ...
	'NumberTitle', 'off', ...
	'BackingStore', 'off', ...
	'Position', figPos, ...
	'CloseRequestFcn', 'm_spat CLOSE;', ...
	'Tag', 'SPAT', ...
	'MenuBar', 'none', ...
	'HandleVisibility', 'callback', ...
	'Pointer', 'crosshair', ...
	'WindowButtonMotionFcn', 'm_spat(''MOVE'',0);', ...
	'WindowButtonDownFcn', 'm_spat DOWN;');

spatState.DEFPOS = figPos;	
			
set(state.SPATWIN, ...
	'Colormap', [hsv(numCols) ; gray(numGrays)], ...
	'Visible', 'on', ...
	'UserData', spatState);
	

%	make menus

set(gui, 'UserData', state);

state = BuildMenu(state.TRAJWIN, state, trajState, spatState);
state = BuildMenu(state.SPATWIN, state, trajState, spatState);
if strcmp(computer,'MAC2'), 
	state = BuildMenu(gui, state, trajState, spatState); 
end;


%	leave GUI on top

figure(gui);


%=============================================================================
%	BUILDMENU  - construct menu structure for specified figure

function state = BuildMenu(fh, state, trajState, spatState)

figure(fh);

%	MAVIS

menu = uimenu(fh, 'Label', 'MAVIS', 'HandleVisibility', 'Callback');
uimenu(menu, 'Label', 'About MAVIS...', ...
	'Callback', 'm_help;');
uimenu(menu, 'Label', 'Open Dataset...', ...
	'Separator', 'on', ...
	'Accelerator', 'O', ...
	'Callback', 'm_gui LOADSET;');
uimenu(menu, 'Label', 'Save Dataset...', ...
	'Accelerator', 'S', ...
	'Callback', 'm_gui SAVESET;');
uimenu(menu, 'Label', 'Export Selection...', ...
	'Accelerator', 'E', ...
	'Callback', 'm_gui SAVESEL;');
h = uimenu(menu, 'Label', 'Import');
uimenu(h, 'Label', 'EVB Dataset...', ...
	'Callback', 'm_gui(''IMPORT'',''EVB'');');
uimenu(h, 'Label', 'Microbeam Dataset...', ...
	'Callback', 'm_gui(''IMPORT'',''MBEAM'');');
uimenu(h, 'Label', 'New MBeam Dataset...', ...
	'Callback', 'm_gui(''IMPORT'',''NEWMB'');');
uimenu(h, 'Label', 'PCM Dataset...', ...
	'Callback', 'm_gui(''IMPORT'',''PCM'');');
uimenu(h, 'Label', 'MAT file variable...', ...
	'Callback', 'm_gui(''IMPORT'',''MAT'');');
uimenu(menu, 'Label', 'Load Configuration...', ...
	'Separator', 'on', ...
	'Callback', 'm_gui LOADCFG;');
uimenu(menu, 'Label', 'Save Configuration...', ...
	'Callback', 'm_gui SAVECFG;');
uimenu(menu, 'Label', 'Load Palate Trace...', ...
	'Separator', 'on', ...
	'Callback', 'm_gui LOADPAL;');
h = uimenu(menu, 'Label', 'Load Image');
uimenu(h, 'Label', 'from Supported Format File...', ...
	'Callback', 'm_gui(''LOADIMG'',''IMG'');');
uimenu(h, 'Label', 'from Microbeam Scan...', ...
	'Callback', 'm_gui(''LOADIMG'',''SCAN'');');
uimenu(h, 'Label', 'from MAT File...', ...
	'Callback', 'm_gui(''LOADIMG'',''MAT'');');
uimenu(menu, 'Label', 'Close MAVIS', ...
	'Separator', 'on', ...
	'Accelerator', 'W', ...
	'Callback', 'm_gui CLOSE;');


%	Trajectory

menu = uimenu(fh, 'Label', 'Trajectory', 'HandleVisibility', 'Callback');
uimenu(menu, 'Label', 'Configure Trajectory Display...', ...
	'Callback', 'm_cfgt INIT;');
backStore = uimenu(menu, 'Label', 'Backing Store', ...
	'Separator', 'on', ...
	'Checked', get(state.TRAJWIN, 'BackingStore'), ...
	'Callback', 'm_traj BSTORE;');
if trajState.PLOTSAMPS, cs = 'on'; else, cs = 'off'; end;
plotSamps = uimenu(menu, 'Label', 'Plot Samples', ...
	'Checked', cs, ...
	'Callback', 'm_traj PLOTSAMPS;');
if trajState.MIDLINES, cs = 'on'; else, cs = 'off'; end;
midlines = uimenu(menu, 'Label', 'Show Midlines', ...
	'Checked', cs, ...
	'Callback', 'm_traj MIDLINES;');
uimenu(menu, 'Label', 'Toggle Background', ...
	'Callback', 'm_traj TOGBG;');
uimenu(menu, 'Label', 'Clone Window', ...
	'Callback', 'm_gui(''CLONE'',''TRAJ'');');
if trajState.AUTOUPDATE, cs = 'on'; else, cs = 'off'; end;
autoMenu = uimenu(menu, 'Label', 'Auto Update', ...
	'Separator', 'on', ...
	'Checked', cs, ...
	'Callback', 'm_traj AUTO;');
uimenu(menu, 'Label', 'Update', ...
	'Accelerator', 'U', ...
	'Callback', 'm_traj UPDATE;');
uimenu(menu, 'Label', 'Full Selection', ...
	'Callback', 'm_traj FULL;');
uimenu(menu, 'Label', 'Yack Selection', ...
	'Separator', 'on', ...
	'Accelerator', 'Y', ...
	'Callback', 'm_traj YACK;');				


%	Spatial

menu = uimenu(fh, 'Label', 'Spatial', 'HandleVisibility', 'Callback');
uimenu(menu, 'Label', 'Set Viewpoint...', ...
	'Callback', 'm_cfgs(''INIT'',''VIEW'');');	
uimenu(menu, 'Label', 'Set Display Options...', ...
	'Callback', 'm_cfgs(''INIT'',''MODE'');');	
uimenu(menu, 'Label', 'Set Linking...', ...
	'Callback', 'm_cfgs(''INIT'',''LINKING'');');	
uimenu(menu, 'Label', 'Set Scaling...', ...
	'Callback', 'm_cfgs(''INIT'',''SCALING'');');
if spatState.INITIALIZED, 
	switch get(spatState.AXIS, 'DrawMode'), 
		case 'normal', cs = 'off';
		case 'fast', cs = 'on';
	end;
else, 
	cs = 'on';
end;
fastRend = uimenu(menu, 'Label', 'Fast Rendering', ...
	'Separator', 'on', ...
	'Checked', cs, ...
	'Callback', 'm_spat RENDER;');
uimenu(menu, 'Label', 'Toggle Background', ...
	'Callback', 'm_spat TOGBG;');
uimenu(menu, 'Label', 'Clone Window', ...
	'Callback', 'm_gui(''CLONE'',''SPAT'');');
uimenu(menu, 'Label', 'Hue Plot Selection', ...
	'Separator', 'on', ...
	'Callback', 'm_spat PLOTSEL;');
if strcmp(computer,'MAC2'), 
	uimenu(menu, 'Label', 'Build QT Movie...', ...
		'Callback', 'm_spat MOVIE;');
end;
uimenu(menu, 'Label', 'Reset View', ...
	'Accelerator', 'R', ...
	'Callback', 'm_spat RESET;');


% 	Movement

menu = uimenu(fh, 'Label', 'Movement', 'HandleVisibility', 'Callback');
uimenu(menu, 'Label', 'Reset to Head', ...
	'Accelerator', 'H', ...
	'Callback', 'm_gui(''MOTION'',''HEAD'');');	
uimenu(menu, 'Label', 'Move Backwards', ...
	'Accelerator', 'B', ...
	'Callback', 'm_gui(''MOTION'',''RREV'');');	
uimenu(menu, 'Label', 'Step Backwards', ...
	'Accelerator', 'K', ...
	'Callback', 'm_gui(''MOTION'',''REV '');');	
uimenu(menu, 'Label', 'Stop Movement', ...
	'Accelerator', 'X', ...
	'Callback', 'm_gui(''MOTION'',''STOP'');');	
uimenu(menu, 'Label', 'Step Forwards', ...
	'Accelerator', 'D', ...
	'Callback', 'm_gui(''MOTION'',''FWD '');');	
uimenu(menu, 'Label', 'Move Forwards', ...
	'Accelerator', 'F', ...
	'Callback', 'm_gui(''MOTION'',''FFWD'');');	
uimenu(menu, 'Label', 'Reset to Tail', ...
	'Accelerator', 'T', ...
	'Callback', 'm_gui(''MOTION'',''TAIL'');');	


% 	Labels

menu = uimenu(fh, 'Label', 'Labels', 'HandleVisibility', 'Callback');
uimenu(menu, 'Label', 'Configure Labelling...', ...
	'Callback', 'm_label(''INIT'',''CONFIG'');');	
if state.LABEL.DISPLAY, cs = 'on'; else, cs = 'off'; end;
showLabs = uimenu(menu, 'Label', 'Show Labels', ...
	'Separator', 'on', ...
	'Accelerator', 'L', ...
	'Checked', cs, ...
	'Callback', 'm_label TOGSHOW;');	
uimenu(menu, 'Label', 'Edit Labels...', ...
	'Callback', 'm_label(''INIT'',''EDIT'');');	
uimenu(menu, 'Label', 'Save Labels...', ...
	'Callback', 'm_gui SAVELABS;');	
uimenu(menu, 'Label', 'Clear All Labels', ...
	'Callback', 'm_label CLEAR;');	

	
% 	Windows

menu = uimenu(fh, 'Label', 'Windows', 'HandleVisibility', 'Callback');
uimenu(menu, 'Label', 'Control Window', ...
	'Accelerator', '1', ...
	'Callback', 'm_gui(''WINDOW'',''GUI'');');	
uimenu(menu, 'Label', 'Trajectory Display', ...
	'Accelerator', '2', ...
	'Callback', 'm_gui(''WINDOW'',''TRAJ'');');	
uimenu(menu, 'Label', 'Spatial Display', ...
	'Accelerator', '3', ...
	'Callback', 'm_gui(''WINDOW'',''SPAT'');');	
uimenu(menu, 'Label', 'Reset Default Positions', ...
	'Separator', 'on', ...
	'Callback', 'm_gui WINDOW;');

	
%	save handles to checked menu items

menus = [autoMenu backStore plotSamps midlines fastRend showLabs];
switch get(fh, 'Tag'),
	case 'TRAJ',
		state.MENUS(1,:) = menus;
	case 'SPAT',
		state.MENUS(2,:) = menus;
	case 'GUI',
		state.MENUS(3,:) = menus;
end;
set(trajState.GUI, 'UserData', state);
