function m_out = m_label(action, aux, aux2, aux3)
%M_LABEL  - MAVIS label support

% Labels are arrays of structs attached to a given dataset trajectory (new
% labels are appended to the end of the current array contents).  While
% user-defined procedures may attach additional fields, MAVIS uses the
% following fields in its default label handlers:
%
%  NAME    - string specifying displayed label name (may be empty)
%  OFFSET  - label offset from time 0 in msecs
%  NOTE    - string specifying any additional information
%  VALUE   - trajectory value at label offset [1 x nDim (dimensions)]
%
% The VALUE field is calculated using the Interpolation Method active at the
% time the label was created (see CONFIGURE TRAJECTORY DISPLAY).

%	branch by action...

switch action,
		
%-----------------------------------------------------------------------------
%	CLEAR:  clear all labels

	case 'CLEAR',
		set(0,'ShowHiddenHandles', 'on');
		gui = findobj('Tag', 'GUI');
		set(0,'ShowHiddenHandles', 'off');
		state = get(gui, 'UserData');
		if isempty(state.DATA), return; end;
		if strcmp(questdlg('Clear all labels...', 'Verify...', 'Yes', 'No', 'Yes'), 'Yes'),
			for t = 1 : length(state.DATA),
				state.DATA(t).LABELS = [];
			end;
			set(gui, 'UserData', state);
			m_traj('INIT', state, 1);
		end;
		
		
%-----------------------------------------------------------------------------
%	CLRSEL:  clear selected labels

	case 'CLRSEL',
		cfgState = get(gcbf, 'UserData');
		state = get(cfgState.GUI, 'UserData');
		killList = get(cfgState.LISTBOX, 'Value');
		if isempty(killList), return; end;
		ki = size(get(cfgState.LISTBOX, 'String'),1);
		for t = length(state.DATA) : -1 : 1,
			for i = length(state.DATA(t).LABELS) : -1 : 1,
				if any(ki == killList),
					state.DATA(t).LABELS(i) = [];
				end;
				ki = ki - 1;
			end;
		end;
		set(cfgState.GUI, 'UserData', state);
		set(cfgState.LISTBOX, 'Value', [], 'String', MakeList(state.DATA));
		set(gcbf, 'Visible', 'off');
		m_traj('INIT', state, 1);
		set(gcbf, 'Visible', 'on');
		
		
%-----------------------------------------------------------------------------
%	CLOSE:  update status if aux nonzero (OK)

	case 'CLOSE',
		if aux > 0,			% OK
			cfgState = get(gcbf, 'UserData');
			state = get(cfgState.GUI, 'UserData');
			switch cfgState.MODE,
				case 'CONFIG',
					file = get(cfgState.PROC, 'String');
					if ~(cfgState.CFGDONE & strcmp(file, cfgState.CFGPROC)),
						if ~isempty(file),
							v = exist(file);
							if ~(v>=2 & v<=6),
								set(warndlg(['Procedure ', file, ' is not in the MATLAB path'], ''), 'WindowStyle', 'modal');
								return;
							end;
						end;
						if ~isempty(file),				% call user proc config handler
							result = feval(file, 'CONFIG', state);
							if ~result, return; end;	% cancelled - don't exit dialog
							state = get(cfgState.GUI, 'UserData');	% retrieve updated state
						end;
					end;
					state.LABEL.PROC = file;
					state.LABEL.ALLTRAJ = get(cfgState.APPLY, 'Value') - 1;
					state.LABEL.ANNOTATE = get(cfgState.ANN, 'Value');
					set(cfgState.GUI, 'UserData', state);
			end;
		end;
		delete(gcbf); 
		
		
%-----------------------------------------------------------------------------
%	EDSEL:  edit selected label

	case 'EDSEL',
		cfgState = get(gcbf, 'UserData');
		state = get(cfgState.GUI, 'UserData');
		if gcbo == cfgState.LISTBOX & ~strcmp(get(gcbf, 'SelectionType'), 'open'), 
			return;				% ignore single listbox clicks
		end;
		edIdx = get(cfgState.LISTBOX, 'Value');	% index of label to edit
		if size(edIdx,2) ~= 1, return; end;		% edit single selection only
		n = 1;
		for t = 1 : length(state.DATA),			% find the label associated with index
			for i = 1 : length(state.DATA(t).LABELS),
				if any(n == edIdx),
					label = state.DATA(t).LABELS(i);
					n = 0;
					break;
				end;
				n = n + 1;
			end;
			if ~n, break; end;
		end;									% Ith label of Tth trajectory
		label = AnnotateLabel(label, state.DATA(t).NAME, 0);
		if isempty(label), return; end;		% cancelled
		label.VALUE = m_gui('GETVAL', state, t, label.OFFSET);
		state.DATA(t).LABELS(i) = label;
		set(cfgState.GUI, 'UserData', state);
		set(gcbf, 'Visible', 'off');
		m_traj('INIT', state, 1);
		set(gcbf, 'Visible', 'on');
		set(cfgState.LISTBOX, 'String', MakeList(state.DATA));
		
		
%-----------------------------------------------------------------------------
%	INITialize:  aux is specifier

	case 'INIT',
		set(0,'ShowHiddenHandles', 'on');
		gui = findobj('Tag', 'GUI');
		set(0,'ShowHiddenHandles', 'off');
		state = get(gui, 'UserData');
		if isempty(state.DATA), return; end;
		figPos = get(0, 'ScreenSize');
		if strncmp(computer, 'SGI', 3), as = 'left'; else, as = 'right'; end;	% SGI bug:  doesn't like right alignment

		switch aux,

% configure labelling
			case 'CONFIG',
				width = 310;
				height = 140;
				figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];
		
				cfg = dialog('Name', 'Configure Labelling', ...
					'Position', figPos, ...
					'ButtonDownFcn', '', ...
					'CreateFcn', 'm_traj(''CURSOR'', ''HIDE'');', ...
					'DeleteFcn', 'm_traj(''CURSOR'', ''SHOW'');', ...
					'CloseRequestFcn', 'm_label(''CLOSE'',0);');
		
				uicontrol(cfg, ...			% user proc
					'Style', 'push', ...
					'Position', [20 98 90 25], ...
					'String', 'Set User Proc', ...
					'Callback', 'm_label SETPROC;');
				proc = uicontrol(cfg, ...
					'Style', 'edit', ...
					'HorizontalAlignment', 'left', ...
					'String', state.LABEL.PROC, ...
					'Position', [120 100 170 20]);
					
				if isunix,
					p1 = [10 60 58 17];
					p2 = [70 58 135 20];
					p3 = [210 60 80 20];
				else,
					p1 = [10 60 55 17];
					p2 = [70 58 150 20];
					p3 = [230 60 80 20];
				end;
				uicontrol(cfg, ...			% apply
					'Position', p1, ...
					'HorizontalAlignment', as, ...
					'String','Apply to:', ...
					'Style','text');
				apply = uicontrol(cfg, ...
					'Position', p2, ...
					'HorizontalAlignment', 'left', ...
					'String','Clicked Trajectory|All Trajectories', ...
					'Value', state.LABEL.ALLTRAJ+1, ...
					'Style','popupmenu');

				ann = uicontrol(cfg, ...	% annotate
					'Position',p3, ...
					'String','Annotate', ...
					'Value', state.LABEL.ANNOTATE, ...
					'Style','checkbox');

				uicontrol(cfg, ...			% buttons
					'Position',[width/2-80 15 60 25], ...
					'String','OK', ...
					'Callback','m_label(''CLOSE'',1);');
				uicontrol(cfg, ...
					'Position',[width/2+20 15 60 25], ...
					'String','Cancel', ...
					'Callback','m_label(''CLOSE'',0);');
			
				cfgState = struct('GUI', gui, ...
								'MODE', aux, ...
								'PROC', proc, ...
								'APPLY', apply, ...
								'ANN', ann, ...
								'CFGDONE', 0, ...
								'CFGPROC', []);
				set(cfg, 'UserData', cfgState);
				
% edit labels
			case 'EDIT',
				width = 500;
				height = 250;
				figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];
		
				cfg = dialog('Name', 'Edit Labels', ...
					'Position', figPos, ...
					'ButtonDownFcn', '', ...
					'CreateFcn', 'm_traj(''CURSOR'', ''HIDE'');', ...
					'DeleteFcn', 'm_traj(''CURSOR'', ''SHOW'');', ...
					'CloseRequestFcn', 'm_label(''CLOSE'',0);');

				switch computer,			% labels
					case 'MAC2', xo = [10 75 165 225];
					case 'PCWIN', xo = [10 95 215 295];
					otherwise, xo = [10 82 186 257];
				end;
				uicontrol(cfg, ...
					'Position', [xo(1) height-31 60 17], ...
					'HorizontalAlignment', 'left', ...
					'String','Trajectory', ...
					'Style','text');
				uicontrol(cfg, ...
					'Position', [xo(2) height-31 40 17], ...
					'HorizontalAlignment', 'left', ...
					'String','Label', ...
					'Style','text');
				uicontrol(cfg, ...
					'Position', [xo(3) height-31 40 17], ...
					'HorizontalAlignment', 'left', ...
					'String','Offset', ...
					'Style','text');
				uicontrol(cfg, ...
					'Position', [xo(4) height-31 40 17], ...
					'HorizontalAlignment', 'left', ...
					'String','Note', ...
					'Style','text');
					
				lb = uicontrol(cfg, ...		% listbox
					'Position',[10 60 width-20 height-90], ...	
					'FontName', 'Courier', ...
					'String', MakeList(state.DATA), ...	
					'Style','listbox', ...
					'Max', 2, ...
					'Value', [], ...
					'UserData', [], ...
					'Callback','m_label EDSEL;');
					
				uicontrol(cfg, ...			% buttons
					'Position',[width/2-120 15 60 25], ...
					'String','Done', ...
					'Callback','m_label(''CLOSE'',0);');
				uicontrol(cfg, ...
					'Position',[width/2-30 15 60 25], ...
					'String','Edit', ...
					'Callback','m_label EDSEL;');
				uicontrol(cfg, ...
					'Position',[width/2+60 15 60 25], ...
					'String','Clear', ...
					'Callback','m_label CLRSEL;');
					
				cfgState = struct('GUI', gui, ...
								'LISTBOX', lb);

% error				
			otherwise,
				error(['M_LABEL:  unknown init option (', aux, ')']);
				
		end;

% save state
		set(cfg, 'UserData', cfgState);
		
	
%-----------------------------------------------------------------------------
%	MAKE:  create label
%
%	returns m_out == 1 on successful creation, m_out == 0 if no label created

	case 'MAKE',
		if nargin < 3,
			trajState = get(gcbf, 'UserData');
			state = get(trajState.GUI, 'UserData');
			import = 0;
		else,
			state = aux2;
			trajState = get(state.TRAJWIN, 'UserData');
			import = 1;
		end;
		if nargin < 2, 		% create default label
			label = struct('NAME', '', 'OFFSET', state.CURSOR, 'NOTE', '', 'VALUE', []);
		else,				% user-supplied label
			label = aux;
			if isempty(label.OFFSET), label.OFFSET = state.CURSOR; end;
		end;
		if import,
			MakeLabel(state, label, 1);
		elseif state.LABEL.ANNOTATE,			% use annotation dialog
			[label, allTraj] = AnnotateLabel(label, state.DATA(trajState.TRACK).NAME, state.LABEL.ALLTRAJ+1);
			if isempty(label),
				m_out = 0;						% cancelled - no label created
				return;
			end;
			MakeLabel(state, label, allTraj);	% create annotated label
		else,
			MakeLabel(state, label);			% create label immediately
		end;
		m_out = 1;			% report label created
		

%-----------------------------------------------------------------------------
% MOVE:  cursor moved (used by user label procs)

	case 'MOVE',
		global fixstr;
		trajState = get(gcbf, 'UserData');
		state = get(trajState.GUI, 'UserData');
	
% get the location of the cursor
		curPt = get(trajState.CURSORH, 'CurrentPoint');
		x = curPt(1,1);
		y = curPt(1,2);

% ignore if out of plotting bounds		
		xLim = get(trajState.CURSORH, 'XLim');			
		yLim = get(trajState.CURSORH, 'YLim');
		if (x < xLim(1)) | (x > xLim(2)) | (y < yLim(1)) | (y > yLim(2)), return; end;

% update GUI field and spatial display if necessary
		oldX = get(trajState.CURSORL, 'XData');
		if x ~= oldX(1),
			set(trajState.CURSORL, 'XData', [x x]);
			state.CURSOR = x;
			set(state.CURSF, 'String', fixstr(x));
			set(trajState.GUI, 'UserData', state);
			m_spat('UPDATE', state);
		end;
		

%-----------------------------------------------------------------------------
%	PLOT:  default label plotting (aux = x, aux2 = y, aux3 = label)

	case 'PLOT',
		x = [aux aux];
		y = aux2 * .9;
		line(x, y);
		if ~isempty(aux3.NAME),
			text(x(2), y(2), [' ', aux3.NAME], ...
				'VerticalAlignment', 'top', ...
				'fontname', 'geneva', ...
				'fontsize', 9);
		end;
		
	
%-----------------------------------------------------------------------------
%	SETPROC:  set user labelling proc

	case 'SETPROC',
		[file, pathName] = uigetfile('ml*.m', 'Select user labelling procedure');
		if file == 0, return; end; 	% cancelled
		cfgState = get(gcbf, 'UserData');
		i = findstr(file, '.');
		if ~isempty(i), file = file(1:i(1)-1); end;
		if ~feval(file, 'CONFIG', get(cfgState.GUI, 'UserData')),	% call user proc config handler
			return; 				% cancelled
		end;
		set(cfgState.PROC, 'String', file);
		cfgState.CFGDONE = 1;
		cfgState.CFGPROC = file;
		set(gcbf, 'UserData', cfgState);

	
%-----------------------------------------------------------------------------
%	TOGSHOW:  toggle label plotting status

	case 'TOGSHOW',
		gui = findobj('Tag', 'GUI');
		state = get(gui, 'UserData');
		trajState = get(state.TRAJWIN, 'UserData');
		state.LABEL.DISPLAY = umtoggle(gcbo);
		set(trajState.GUI, 'UserData', state);
		if state.LABEL.DISPLAY, s = 'on'; else, s = 'off'; end;
		set(state.MENUS(:,6), 'Checked', s);
		if ~trajState.INITIALIZED, return; end;	
		m_traj('INIT', state, 1);


%-----------------------------------------------------------------------------
%	error

	otherwise,
		error(['M_LABEL:  unrecognized action (', action, ')']);
	
end;


%=============================================================================
% AnnotateLabel  - label annotation dialog
%
%	label		- initialized label (for field editing)
%	trajName	- clicked trajectory name
%	apply		- == 0 disable apply menu, > 0 value code for menu
%
% returns updated label, allTraj on OK, [] on cancel

function [label, allTraj] = AnnotateLabel(label, trajName, apply)

global fixstr;

figPos = get(0, 'ScreenSize');

width = 280;
height = 170;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];
if strncmp(computer, 'SGI', 3), as = 'left'; else, as = 'right'; end;	% SGI bug:  doesn't like right alignment

cfg = dialog('Name', 'Create Label', ...
	'Position', figPos, ...
	'ButtonDownFcn', '', ...
	'UserData', 0);

uicontrol(cfg, ...		% name
	'Position', [10 135 40 17], ...
	'HorizontalAlignment', as, ...
	'String','Name:', ...
	'Style','text');
name = uicontrol(cfg, ...
	'Position',[55 135 85 20], ...
	'Style','edit', ...
	'String', label.NAME, ...
	'HorizontalAlignment', 'left');

uicontrol(cfg, ...		% offset
	'Position', [145 135 50 17], ...
	'HorizontalAlignment', as, ...
	'String','Offset:', ...
	'Style','text');
offset = uicontrol(cfg, ...
	'Position',[200 135 60 20], ...
	'Style','edit', ...
	'String', fixstr(label.OFFSET), ...
	'HorizontalAlignment', 'left');

uicontrol(cfg, ...		% note
	'Position', [10 100 40 17], ...
	'HorizontalAlignment', as, ...
	'String','Note:', ...
	'Style','text');
if ischar(label.NOTE) | isempty(label.NOTE), s = label.NOTE; else, s = '(data)'; end;
note = uicontrol(cfg, ...
	'Position',[55 100 205 20], ...
	'Style','edit', ...
	'String', s, ...
	'HorizontalAlignment', 'left');

uicontrol(cfg, ...		% apply
	'Position', [30 62 55 17], ...
	'HorizontalAlignment', as, ...
	'String','Apply to:', ...
	'Style','text');
s = [trajName, '|All Trajectories'];
if apply, v = apply; e = 'on'; else, v = 1; e = 'off'; end;
applyTo = uicontrol(cfg, ...
	'Position', [90 60 150 20], ...
	'HorizontalAlignment', 'left', ...
	'String', s, ...
	'Value', v, ...
	'Enable', e, ...
	'Style','popupmenu');

% OK, cancel buttons
uicontrol(cfg, ...		% buttons
	'Position',[width/2-70 15 60 25], ...
	'String','OK', ...
	'Callback','set(gcbf,''UserData'',1);uiresume');
uicontrol(cfg, ...
	'Position',[width/2+10 15 60 25], ...
	'String','Cancel', ...
	'Callback','uiresume');

% wait for input
uiwait(cfg);
if get(cfg, 'UserData'),
	label.NAME = get(name, 'String');
	label.OFFSET = str2num(get(offset, 'String'));
	if ischar(label.NOTE) | isempty(label.NOTE), label.NOTE = get(note, 'String'); end;
	allTraj = get(applyTo, 'Value') - 1;
else,
	label = [];
	allTraj = [];
end;
delete(cfg);


%=============================================================================
%	MakeLabel  - attach label to trajectory

function MakeLabel(state, label, allTraj)

%	default arguments
if nargin < 3, allTraj = state.LABEL.ALLTRAJ; end;

%	apply label to...
trajState = get(state.TRAJWIN, 'UserData');		% clicked trajectory

if allTraj,
	apply = 1 : length(state.DATA);
else,
	apply = trajState.TRACK;
end;
for t = apply,
	label.VALUE = m_gui('GETVAL', state, t, label.OFFSET);
	if isempty(state.DATA(t).LABELS),
		state.DATA(t).LABELS = label;
	else,
		state.DATA(t).LABELS(end+1) = label;
	end;
end;

% plot the label
if allTraj,
	apply = [1:length(find(ismember([1:length(state.DATA)], trajState.DISPMAP)))];
else,
	apply = find(trajState.TRACK == trajState.DISPMAP);
end;
for t = apply,
	axes(trajState.PANELS(t+1));				
	y = get(trajState.PANELS(t+1), 'YLim');
	x = label.OFFSET * state.DATA(trajState.DISPMAP(t)).SRATE/1000 + 1;
	if isempty(state.LABEL.PROC),	% default plotting
		m_label('PLOT', x, y, label);
	else,							% user proc
		feval(state.LABEL.PROC, 'PLOT', x, y, label);
	end;
end;

set(trajState.GUI, 'UserData', state);


%	reorder axes handles
for p = 2 : length(trajState.PANELS),
	axes(trajState.PANELS(p));
end;
axes(trajState.CURSORH);


%=============================================================================
%	MakeList  - build label listbox contents

function labels = MakeList(data)

labels = {};

for t = 1 : length(data),
	for i = 1 : length(data(t).LABELS),
		label = data(t).LABELS(i);
		if ischar(label.NOTE) | isempty(label.NOTE), sn = label.NOTE; else, sn = '(data)'; end;
		labName = label.NAME;
		if isempty(labName), labName = ' '; end;
		s = sprintf('%-10s%-15s%-10g%s', ...
			data(t).NAME, ...
			labName, ...
			label.OFFSET, ...
			sn);
		if isempty(labels),
			labels = {s};
		else,
			labels{end+1} = s;
		end;
	end;
end;
