function m_spat(action, aux)
%M_SPAT  - MAVIS spatial window support

%	branch by action...

switch action,

%-----------------------------------------------------------------------------
%	CLOSE:  trap close window requests

	case 'CLOSE',
		m_gui('CLOSE');
					
	
%-----------------------------------------------------------------------------
%	DOWN:  mouse down

	case 'DOWN',
		spatState = get(gcbf, 'UserData');
		if ~spatState.INITIALIZED, return; end;
		switch get(gcbf, 'SelectionType'),
			case 'normal',		% normal mouse click
				spatState.AUTOSCALE = 0;	
				curPt = mean(get(spatState.AXIS, 'CurrentPoint'));
				bounds = [get(spatState.AXIS, 'Xlim')' get(spatState.AXIS, 'Ylim')' get(spatState.AXIS, 'Zlim')'];
				if spatState.FLAT, curPt(3) = bounds(1,3); end;
				if curPt(1)<bounds(1,1) | curPt(1)>bounds(2,1) | ...
						curPt(2)<bounds(1,2) | curPt(2)>bounds(2,2) | ...
						curPt(3)<bounds(1,3) | curPt(3)>bounds(2,3),
					return;		% out of bounds
				end;
				spatState.OLDPT = curPt;
				set(gcbf, 'UserData', spatState, ...
					'WindowButtonMotionFcn', 'm_spat(''MOVE'',1);', ...
					'WindowButtonUpFcn', 'm_spat(''UP'', 0);');
			case 'open',		% double click - set default view
				ResetView(get(gcbf, 'UserData'));
			otherwise,			% option or shift mouse click (mac); right button click
				if spatState.FLAT, return; end;		% rotation disabled if image displayed
				rotate3d on;	% change azimuth, elevation interactively
				set(gcbf, 'WindowButtonUpFcn', 'm_spat(''UP'', 1);');
				rotate3d down;
		end;
	
	
%-----------------------------------------------------------------------------
%	INITialize:  aux holds (gui) state

	case 'INIT',
		global L_OFF L_OPEN L_CLOSE L_NSPLINE D_BLOBS D_NOERASE D_VELOCITY D_GHOSTS HISTLEN;

		state = aux;
		spatState = get(state.SPATWIN, 'UserData');
		state = get(spatState.GUI, 'UserData');		% refresh for m_traj() changes
		trajState = get(state.TRAJWIN, 'UserData');
		
		set(state.SPATWIN, 'HandleVisibility', 'on');
		figure(state.SPATWIN);
		if spatState.INITIALIZED,
			[az, el] = view;							% remember view
			drawMode = get(spatState.AXIS, 'DrawMode');	% and render mode
		else,
			drawMode = 'fast';
		end;
		
% find plotting bounds
		ranges = [inf -inf ; inf -inf ; inf -inf ];		% X, Y, Z
		nComp = 0;
		plotMap = trajState.DISPMAP;
		for t = 1 : length(plotMap),	
			if strcmp(state.DATA(plotMap(t)).SCALING.TYPE, 'COMMON'),
				for c = 1 : size(state.DATA(plotMap(t)).SIGNAL,2),
					ranges(c,1) = min([ranges(c,1) state.DATA(plotMap(t)).SCALING.RANGE(c,1)]);
					ranges(c,2) = max([ranges(c,2) state.DATA(plotMap(t)).SCALING.RANGE(c,2)]);
				end;
				if c > nComp, nComp = c; end;
			else,
				plotMap(t) = 0;
			end;
		end;
		plotMap = plotMap(find(plotMap>0));
		nTraj = length(plotMap);
		if ~nTraj,
			clf;
			spatState.INITIALIZED = 0;		% nothing to plot
			set(state.SPATWIN, 'UserData', spatState, 'HandleVisibility', 'callback');
			return;
		end;
		ranges(isinf(ranges)) = 0;
		if spatState.AUTOSCALE,
			spread = max(ranges(:,2) - ranges(:,1));
			spread = spread + spread * 0.2;		% squish
			spatState.OFFSETS = [0 0 0];
		else,
			spread = spatState.SPREAD;
		end;
		ranges(:,1) = ranges(:,1) + (ranges(:,2)-ranges(:,1)-spread)/2 - spatState.OFFSETS';
		ranges(:,2) = ranges(:,1) + spread;

% compute max (Euclidean) velocity across plotted trajectories
		maxVel = 0;
		if spatState.MODE == D_VELOCITY,		
			for t = plotMap,
				maxVel = max([maxVel; sqrt(sum(diff(state.DATA(t).SIGNAL).^2,2))]);
			end;
			if ~maxVel, maxVel = 1; end;
		end;

% separate handling depending on data dimensionality
		clf;				% clear axes
		if nComp < 2 | nComp > 3,		% mono or other dimensional data
			spatState.PLOTMAP = [];
			set(state.SPATWIN, 'UserData', state, 'HandleVisibility', 'callback');
			return;	
		end;			
		
% special treatment if image displayed
		if spatState.IMAGE.DISPLAY & ~isempty(spatState.IMAGE.DATA),
			idp = size(spatState.IMAGE.DATA);		% image dimensions in pixels
			idm = idp / spatState.IMAGE.PIXPERMM;	% in mm

			ranges(1,1) = ranges(1,1) + (spread - idm(2)) / 2;
			ranges(1,2) = ranges(1,1) + idm(2);		% x axis range
			ranges(2,1) = ranges(2,1) + (spread - idm(1)) / 2;
			ranges(2,2) = ranges(2,1) + idm(1);		% y axis range
			spread = max(idm);
			
% scale image to upper colormap (grayscale) range
			if get(0, 'ScreenDepth') < 16,
				img = flipud(uint8(.125 * double(spatState.IMAGE.DATA) + 16));	% [hsv(16) ; gray(32)]
			else,
				img = flipud(uint8(.25 * double(spatState.IMAGE.DATA) + 64));	% [hsv(64) ; gray(64)]
			end;

			h = image(ranges(1,:), ranges(2,:), img);	% creates imgAxis
			imgAxis = gca;
			
			aPos = get(imgAxis, 'Position');
			fPos = get(state.SPATWIN, 'Position');
			dx = ceil(idp(2)/aPos(3) - fPos(3));
			dy = ceil(idp(1)/aPos(4) - fPos(4));
			fPos = [fPos(1)-round(dx/2) fPos(2)-round(dy/2) fPos(3)+dx fPos(4)+dy];
			set(state.SPATWIN, 'Position', fPos);	% resize spatial window to accommodate image
			dx = idp(2)/fPos(3) - aPos(3);
			dy = idp(1)/fPos(4) - aPos(4);
			aPos = [aPos(1)-dx/2 aPos(2)-dy/2 aPos(3)+dx aPos(4)+dy];
			set(imgAxis, 'Position', aPos, ...		% resize axis for image truesize
					'Ydir', 'normal');
			axis off;
			axis equal;
			spatAxis = axes('Position', aPos, 'drawmode', drawMode);	% create plotting axis
		else,
			imgAxis = [];
			spatAxis = axes('drawmode', drawMode);
		end;

		axis(reshape(ranges',1,6));

		if isempty(imgAxis),
% 			switch nComp,
% 				case 2,			% 2D
% 					if spatState.INITIALIZED, view(az,el); else, view(2); end;
% 				case 3,			% 3D
% 					if spatState.INITIALIZED, view(az,el); else, view(3); end;
% 			end;
			if spatState.INITIALIZED, view(az,el); else, view(2); end;
		else,
			set(spatAxis, 'color', 'none');
			view(2);
		end;
			
		
% plot palate trace
		if spatState.PALATE.DISPLAY & ~isempty(spatState.PALATE.DATA),
			switch nComp,
				case 2,
					line(spatState.PALATE.DATA(:,1), spatState.PALATE.DATA(:,2));
					if ~isempty(spatState.PALATE.PHA),		% mbeam pha supprt
						line(spatState.PALATE.PHA(:,1), spatState.PALATE.PHA(:,2));
						if spatState.AUTOSCALE,		
							ranges(1,1) = min(spatState.PALATE.PHA(:,1))-5;
							axis(reshape(ranges',1,6));
							view(2);
						end;
					end;
				case 3,
					if size(spatState.PALATE.DATA,2<3),
						line(spatState.PALATE.DATA(:,1), spatState.PALATE.DATA(:,2), zeros(length(spatState.PALATE.DATA),1));
					else,
						line(spatState.PALATE.DATA(:,1), spatState.PALATE.DATA(:,2), spatState.PALATE.DATA(:,3));
					end;
				end;
		end;

% plot inter-blobular lines
		if spatState.LINKMODE > L_OFF & length(spatState.LINKMAP) > 1,
			linkMap = spatState.LINKMAP;
			p = zeros(length(linkMap), nComp);
			for i = 1 : length(linkMap),
				p(i,:) = m_gui('GETVAL', state, linkMap(i), state.CURSOR);
			end;
			switch spatState.LINKMODE,
				case L_OPEN,		% lines connect points
				case L_CLOSE,		% lines connect points (as closed set)
					p(end+1,:) = p(1,:);
				case L_NSPLINE,		% natural cubic spline interpolant
					if length(linkMap) > 2, p = csplint(p); end;
			end;
			switch nComp,
				case 2,
					links = line(p(:,1), p(:,2), ...
								'EraseMode', 'xor', ...
								'Color', [.5 .5 .5]);
				case 3,
					links = line(p(:,1), p(:,2), p(:,3), ...
								'EraseMode', 'xor', ...
								'Color', [.5 .5 .5]);
			end;
		else,
			links = [];
		end;

% plot blobs
		map = get(state.SPATWIN, 'colormap');
		if get(0,'ScreenDepth') < 16, numCols = 16; else, numCols = 64; end;
		md = numCols /nTraj;
		
		history = [];
		switch spatState.MODE,
			case D_BLOBS,	eraseMode = 'xor';
			case D_NOERASE, eraseMode = 'none';
			case D_VELOCITY, eraseMode = 'xor';
			case D_GHOSTS, 	
				history = zeros(length(plotMap), HISTLEN+1); 
				eraseMode = 'xor';
				isBlack = get(state.TRAJWIN, 'Color') < .5;
		end;
		
		ci = 1;		% color index
		for i = 1 : nTraj,
			color = map(floor((ci-1)*md)+1,:);
			ci = ci + 1;
			p = m_gui('GETVAL', state, plotMap(i), state.CURSOR);
			
			if spatState.MODE == D_GHOSTS,		% build history trail
				if isBlack, 
					colors = [color ; [0 0 0]]; 
				else, 
					colors = [color ; [1 1 1]]; 
				end;
				colors = interp1([1 2], colors, [1:1/HISTLEN:2], '*linear');
				for n = HISTLEN+1 : -1 : 2,
					switch nComp,
						case 2,
							history(i, n) = line(p(1), p(2), ...
								'Color', colors(n,:), ...
								'Marker', spatState.BLOBTYPE, ...
								'MarkerSize', spatState.BLOBSIZE, ...
								'EraseMode', eraseMode);
						case 3,
							history(i, n) = line(p(1), p(2), p(3), ...
								'Color', colors(n,:), ...
								'Marker', spatState.BLOBTYPE, ...
								'MarkerSize', spatState.BLOBSIZE, ...
								'EraseMode', eraseMode);
					end;	
				end;
			end;
			
			switch nComp,
				case 2,
					blobs(i) = line(p(1), p(2), ...
						'Color', color, ...
						'Marker', spatState.BLOBTYPE, ...
						'MarkerSize', spatState.BLOBSIZE, ...
						'MarkerFaceColor', color, ...
						'EraseMode', eraseMode);
				case 3,
					blobs(i) = line(p(1), p(2), p(3), ...
						'Color', color, ...
						'Marker', spatState.BLOBTYPE, ...
						'MarkerSize', spatState.BLOBSIZE, ...
						'MarkerFaceColor', color, ...
						'EraseMode', eraseMode);
			end;	
			
			if spatState.MODE == D_GHOSTS, history(i, 1) = blobs(i); end;
			
		end;
		if isempty(imgAxis), axis square; end;
		box on;
		if spatState.GRID, grid on; end;
		xlabel('X'); ylabel('Y'); zlabel('Z');
		
% update trajectory window internal state
		spatState.AXIS = spatAxis;			% plotting axis
		spatState.PLOTMAP = plotMap;		% indices of plotted trajectories (into state.DATA)
		spatState.BLOBS = blobs;			% plotted pellet handles
		spatState.LINKS	= links;			% connecting lines
		spatState.SPREAD = spread;			% plotting spread
		spatState.FLAT = ~isempty(imgAxis);	% flat view only (if image)
		spatState.HISTORY = history;		% init history trail
		spatState.MAXVEL = maxVel;			% max velocity of plotted data
		spatState.INITIALIZED = 1;			% initialization complete
		
		set(state.SPATWIN, 'UserData', spatState, 'HandleVisibility', 'callback');
		
		
%-----------------------------------------------------------------------------
%	MOVE:  mouse motion

	case 'MOVE',
		if ~strncmp(get(gcf, 'Tag'), 'SPAT', 1), return; end;
		spatState = get(gcbf, 'UserData');
		if ~spatState.INITIALIZED, return; end;
		state = get(spatState.GUI, 'UserData');
		curPt = mean(get(spatState.AXIS, 'CurrentPoint'));
		bounds = [get(spatState.AXIS, 'Xlim')' get(spatState.AXIS, 'Ylim')' get(spatState.AXIS, 'Zlim')'];
		if spatState.FLAT, curPt(3) = bounds(1,3); end;
		if curPt(1)<bounds(1,1) | curPt(1)>bounds(2,1) | ...
				curPt(2)<bounds(1,2) | curPt(2)>bounds(2,2) | ...
				curPt(3)<bounds(1,3) | curPt(3)>bounds(2,3),
			inBounds = 0;
		else,
			inBounds = 1;
		end;
		global fixstr;
		switch aux,
			case 0,		% update position
				if inBounds,
					set(state.XVALF, 'String', fixstr(curPt(1)));
					set(state.YVALF, 'String', fixstr(curPt(2)));
					set(state.ZVALF, 'String', fixstr(curPt(3)));
				else,
					set(state.XVALF, 'String', '');
					set(state.YVALF, 'String', '');
					set(state.ZVALF, 'String', '');
				end;
			case 1,		% change offset
				d = curPt - spatState.OLDPT;
				spatState.OLDPT = curPt - d;
				spatState.OFFSETS = spatState.OFFSETS + d;
				set(gcbf, 'UserData', spatState);
				s = 'XYZ';
				for i = 1 : 3,
					range = get(spatState.AXIS, [s(i) 'lim']);
					range = range - d(i);
					set(spatState.AXIS, [s(i) 'lim'], range);
				end;
				set(state.XVALF, 'String', fixstr(spatState.OFFSETS(1)));
				set(state.YVALF, 'String', fixstr(spatState.OFFSETS(2)));
				set(state.ZVALF, 'String', fixstr(spatState.OFFSETS(3)));
		end;
		
		
%-----------------------------------------------------------------------------
%	MOVIE:  build QuickTime movie (Mac only)

	case 'MOVIE',
		global msecsToSamps;
		h = findobj('Tag', 'SPAT');
		spatState = get(h, 'UserData');
		if ~spatState.INITIALIZED, return; end;
		state = get(spatState.GUI, 'UserData');
		figure(state.SPATWIN);
		map = get(state.SPATWIN, 'colormap');

% QT options:  frame rate, animation compression, medium quality
		fRate = 30; comp = 3; quality = 3;
		
% make the Matlab movie
		nFrames = round((state.TAIL - state.HEAD) * fRate / 1000);
		dt = 1000/fRate;
		mov = moviein(nFrames, state.SPATWIN);
		cursor = state.CURSOR;
		state.CURSOR = state.HEAD;
		m_spat('UPDATE', state);
		mov(:,1) = getframe(state.SPATWIN);
		for f = 2 : nFrames,
			state.CURSOR = state.CURSOR + dt;
			m_spat('UPDATE', state);
			mov(:,f) = getframe(state.SPATWIN);
		end;
		state.CURSOR = cursor;
		m_spat('UPDATE', state);

% write video track

		setName = get(state.NAMEF, 'String');
		[file, pathName] = uiputfile([setName '.mov'], 'Save QT movie as');
		if file == 0, return; end;		% cancelled
		fileName = [pathName, file];
		qtwrite(mov, map, fileName, [fRate, comp, quality]);
		clear mov map;

% write audio track
		if size(state.DATA(1).SIGNAL,2) == 1 & state.DATA(1).SRATE >= 500,
			head = msecsToSamps(state.HEAD, state.DATA(1).SRATE);
			tail = msecsToSamps(state.TAIL, state.DATA(1).SRATE);
			[pathName, file] = fileparts(fileName);
			sndName = [pathName file '.snd'];
			writesnd(state.DATA(1).SIGNAL(head:tail,1), state.DATA(1).SRATE, 16, sndName);
		end;

% launch movie, squawk sound file
		applescript('RunAndSquawk', 'movieName', ['"' fileName '"'], 'sndName', ['"' sndName '"']);

	
%-----------------------------------------------------------------------------
%	PLOTSEL:  plot current temporal selection with history coloring

	case 'PLOTSEL',
		global msecsToSamps;
		h = findobj('Tag', 'SPAT');
		spatState = get(h, 'UserData');
		if ~spatState.INITIALIZED, return; end;
		state = get(spatState.GUI, 'UserData');
		figure(state.SPATWIN);

		if get(0,'ScreenDepth') < 16, nCols = 16; else, nCols = 64; end;
		map = get(state.SPATWIN, 'colormap');	
		for i = spatState.PLOTMAP,
			clear data;
			h = msecsToSamps(state.HEAD, state.DATA(i).SRATE);
			t = msecsToSamps(state.TAIL, state.DATA(i).SRATE);
			data = state.DATA(i).SIGNAL(h:t,:);
			[nSamps, nComps] = size(data);
			ci = round(interp1([1:nCols],[1:(nCols-1)/(nSamps-1):nCols],'*linear'));
			h = 1; 
			while h < nSamps,
				t = find(ci == ci(h));
				if isempty(t),
					nComps = 0;
				else,
					t = t(end);
					if h == t, t = h + 1; end;
				end;
				switch nComps,
					case 0,
						t = nSamps;
					case 2,
						line(data(h:t,1), data(h:t,2), ...
								'Color', map(ci(h),:), ...
								'LineWidth', 1.5);
					case 3,
						line(data(h:t,1), data(h:t,2), data(h:t,3), ...
								'Color', map(ci(h),:), ...
								'LineWidth', 1.5);
				end;
				h = t;
			end;
		end;
		
		
%-----------------------------------------------------------------------------
%	RENDER:  toggle axes drawing mode status

	case 'RENDER',
		spatWin = findobj('Tag', 'SPAT');
		spatState = get(spatWin, 'UserData');
		if ~spatState.INITIALIZED, return; end;
		newVal = umtoggle(gcbo);
		if newVal, s = 'on'; m = 'fast'; else, s = 'off'; m = 'normal'; end;
		state = get(spatState.GUI, 'UserData');
		set(state.MENUS(:,5), 'Checked', s);
		set(spatState.AXIS, 'DrawMode', m);


%-----------------------------------------------------------------------------
%	RESET:  reset view

	case 'RESET',
		set(0,'ShowHiddenHandles', 'on');
		h = findobj('Tag', 'SPAT');
		set(0,'ShowHiddenHandles', 'off');
		spatState = get(h, 'UserData');
		if spatState.INITIALIZED, ResetView(spatState); end;
		
		
%-----------------------------------------------------------------------------
%	TOGBG:  toggle background color

	case 'TOGBG',
		spatWin = findobj('Tag', 'SPAT');
		spatState = get(spatWin, 'UserData');
		figure(spatWin);
		if any(get(spatWin, 'Color') < .5), newBGC = 'white'; else, newBGC = 'black'; end;
		map = get(spatWin, 'colormap');
		clf;
		colordef(spatWin, newBGC);
		set(spatWin, 'colormap', map);
		if spatState.INITIALIZED,
			spatState.INITIALIZED = 0;
			set(spatWin, 'UserData', spatState);
			state = get(spatState.GUI, 'UserData');
			set(state.MENUS(:,5), 'Checked', 'on');
			m_spat('INIT', state); 
		end;
		

%-----------------------------------------------------------------------------
%	UP:  mouse up

	case 'UP',
		if aux,				% rotate3d in progress
			rotate3d up;
			rotate3d off;
		else,				% offset change in progress
			set(gcbf, 'WindowButtonMotionFcn', 'm_spat(''MOVE'',0);');
			gui = findobj('Tag', 'GUI');
			state = get(gui, 'UserData');
			h = [state.XVALF state.YVALF state.ZVALF];
			set(h, 'String', '');
		end;
	
%-----------------------------------------------------------------------------
%	UPDATE:  plot new position (aux holds state)

	case 'UPDATE',
		global L_OPEN L_CLOSE L_NSPLINE D_VELOCITY D_GHOSTS HISTLEN;

		state = aux;
		spatState = get(state.SPATWIN, 'UserData');
		if ~spatState.INITIALIZED, return; end;
		nComp = size(state.DATA(spatState.PLOTMAP(1)).SIGNAL,2);

% update link lines		
		if spatState.LINKS,						
			linkMap = spatState.LINKMAP;
			p = zeros(length(linkMap), nComp);
			for i = 1 : length(linkMap),
				p(i,:) = m_gui('GETVAL', state, linkMap(i), state.CURSOR);
			end;
			switch spatState.LINKMODE,
				case L_OPEN,		% lines connect points
				case L_CLOSE,		% lines connect points (as closed set)
					p(end+1,:) = p(1,:);
				case L_NSPLINE,		% natural cubic spline interpolant
					if length(linkMap) > 2, p = csplint(p); end;
			end;
			switch nComp,
				case 2,
					set(spatState.LINKS, ...
						'XData', p(:,1), ...
						'YData', p(:,2));
				case 3,
					set(spatState.LINKS, ...
						'XData', p(:,1), ...
						'YData', p(:,2), ...
						'ZData', p(:,3));
			end;			
		end;

% update pellet locs
		for i = 1 : length(spatState.PLOTMAP),	
			p = m_gui('GETVAL', state, spatState.PLOTMAP(i), state.CURSOR);
			

			switch spatState.MODE,

% get previous point for velocity basis
				case D_VELOCITY,
					lastPos = [get(spatState.BLOBS(i),'XData'), get(spatState.BLOBS(i),'YData'), get(spatState.BLOBS(i),'ZData')];
			
% update history if tracking ghosts
				case D_GHOSTS,
					history = spatState.HISTORY;
					for n = HISTLEN+1 : -1 : 2,
						set(history(i, n), 'XData', get(history(i, n-1), 'XData'));
						set(history(i, n), 'YData', get(history(i, n-1), 'YData'));
						if nComp>2, set(history(i, n), 'ZData', get(history(i, n-1), 'ZData')); end;
					end;
			end;

% blobular update
			switch nComp,
				case 2,
					set(spatState.BLOBS(i), ...
						'XData', p(1), ...
						'YData', p(2));
				case 3,
					set(spatState.BLOBS(i), ...
						'XData', p(1), ...
						'YData', p(2), ...
						'ZData', p(3));
			end;
			
% update pellet size if tracking velocity
			if spatState.MODE == D_VELOCITY,
				if state.MOTION,	% if moving
					dp = lastPos - p;
				else,				% not moving - use MINSTEP delta (1 sample for slowest trajectory)
					dp = m_gui('GETVAL', state, spatState.PLOTMAP(i), state.CURSOR-state.MINSTEP) - p;
				end;
				spatState.HISTORY(i,:) = p;
				v = 2*sqrt(sum(dp.^2))/spatState.MAXVEL + 1;		% maxVel is thrice blob size
				if v > 3, v = 3; end								% minimize head : tail flash
				set(spatState.BLOBS(i), 'MarkerSize', v*spatState.BLOBSIZE);
			end;
			
		end;

	
%-----------------------------------------------------------------------------
%	error

	otherwise,
		error(['M_SPAT:  unrecognized action (', action, ')']);
	
end;


%=============================================================================
%	csplint  - cubic spline interpolation; natural end conditions

function ip = csplint(p)

res = 10;				% number of interpolated points

[np, nd] = size(p);		% number of points, number of dimensions

% set knot spacing to Euclidean distance between points
knots(1) = 0;
for i = 2 : np,
	delta = p(i,:) - p(i-1,:);
	knots(i) = knots(i-1) + sqrt(sum(delta .^ 2));
end;

% parametric offsets for spline evaluation
t = knots(1);	
for i = 2 : length(knots),
	delta = (knots(i) - knots(i-1)) / res;
	t = [t [knots(i-1)+delta : delta : knots(i)]];
end;
t = t(2:end)';
st = size(t);

%	loop:  for each dimension

ip = zeros((np-1)*res+1,nd);
for i = 1 : nd,

%	compute 2nd derivatives of interpolating function at knot points (Ay = b)

	h = diff(knots)';
	dydh = diff(p(:,i)) ./ h;
	A = diag(h(2:np-2),-1) + 2*diag(h(2:np-1)+h(1:np-2)) + diag(h(2:np-2),1);
 	b = 6*(dydh(2:np-1) - dydh(1:np-2));
	
 	dy2 = [0 ; A\b ; 0];			% natural spline
	
%	evaluate at parametric offsets

	hh = reshape(repmat(h',res,1), st);
	hh2 = reshape(repmat((h.*h)',res,1),st);
	a = (reshape(repmat(knots(2:end),res,1),st) - t) ./ hh;
	b = (t - reshape(repmat(knots(1:end-1),res,1),st)) ./ hh;
	yhi = reshape(repmat(p(2:end,i)',res,1),st);
	ylo = reshape(repmat(p(1:end-1,i)',res,1),st);
	dy2hi = reshape(repmat(dy2(2:end)',res,1),st);
	dy2lo = reshape(repmat(dy2(1:end-1)',res,1),st);
	ip(:,i) = [p(1,i) ; a.*ylo + b.*yhi + ((a.^3-a).*dy2lo + (b.^3-b).*dy2hi).*hh2./6];

end;


%=============================================================================
%	ResetView  - reset default view

function ResetView(spatState)

state = get(spatState.GUI, 'UserData');
set(state.SPATWIN, 'HandleVisibility', 'on');
nComp = size(state.DATA(spatState.PLOTMAP(1)).SIGNAL,2);
if isempty(spatState.PREFVIEW),
%	view(nComp);
	view(2);
else,
	view(spatState.PREFVIEW);
end;
s = 'XYZ';
for i = 1 : 3,
	range = get(spatState.AXIS, [s(i) 'lim']);
	range = range + spatState.OFFSETS(i);
	set(spatState.AXIS, [s(i) 'lim'], range);
	set(spatState.AXIS, [s(i) 'dir'], 'normal');
end;
spatState.OFFSETS = [0 0 0];
spatState.AUTOSCALE = 1;
set(state.SPATWIN, 'UserData', spatState);
set(state.SPATWIN, 'HandleVisibility', 'off');
m_spat('INIT', get(spatState.GUI, 'UserData'));
