function m_out = ml_colors(action, aux, aux2, aux3)
%ML_COLORS  - MAVIS user labelling procedure
%
% Works like default MAVIS label handlers, except that the contents of
% the label NOTE field are interpreted as the plotting color and style
% of the label (e.g. r: will plot as a red dotted line).  An RGB color
% triplet can also be used (e.g. [.5 .5 .5]).  Empty or non-string
% NOTE fields produce a standard label. 

% 01/98 mkt

%	branch by action...

switch action,
		
%-----------------------------------------------------------------------------
% CONFIG:  handle configuration
%
%	AUX		- internal state
%
% no configuration necessary, just show blurb

	case 'CONFIG',
		ShowBlurb;
		m_out = 1;					% report OK
		return;
		
		
%-----------------------------------------------------------------------------
% DOWN:  handle mouseDown
%
%	AUX		- internal state
%	AUX2	- index of clicked trajectory (into state.DATA)

	case 'DOWN',
		state = aux;
		set(state.TRAJWIN,  ...			% init motion handlers
			'WindowButtonMotionFcn', 'm_label MOVE;', ...
			'WindowButtonUpFcn', 'ml_colors UP;');


%-----------------------------------------------------------------------------
% PLOT:  plot label into the current axes
%
%	AUX		- x offset (in trajectory sample units) of label
%	AUX2	- yLim of current axis
%	AUX3	- label

	case 'PLOT',
		x = [aux aux];
		y = aux2 * .9;
		label = aux3;

% NOTE field empty or not a string -- plot default label
		if isempty(label.NOTE) | ~ischar(label.NOTE),
			line(x, y);					

% NOTE field specifies color triplet
		elseif label.NOTE(1) == '[',
			[c, nc] = sscanf(label.NOTE, '[%f %f %f]');
			if nc == 3,
				line(x, y, 'Color', c);
			else,
				line(x, y);		% if invalid
			end;
			
% NOTE field specifies color by character
		else,
			colors = get(gca, 'ColorOrder');
			whiteBG = (colors(1,3) == 1);
			c = findstr(label.NOTE(1), 'ymcrgbwk');
			if isempty(c),
				color = colors(1,:);
			elseif c == 7,
				color = [1 1 1];
			elseif c == 8,
				color = [0 0 0];
			elseif whiteBG,
				color = colors(7-c,:);
			else,
				color = colors(c,:);
			end;
			if length(label.NOTE) > 1,
				switch label.NOTE(2),
					case '-',
						test = [label.NOTE(2:end) ' '];
						switch test(2),
							case '.', style = '-.';
							case '-', style = '--';
							otherwise, style = '-';
						end;
					case ':',
						style = ':';
					otherwise,
						style = '-';
				end;
			else,
				style = '-';
			end;
			line(x, y, 'Color', color, 'LineStyle', style);			
		end;

		if ~isempty(label.NAME),
			text(x(2), y(2), [' ', label.NAME], ...
				'VerticalAlignment', 'top', ...
				'fontname', 'geneva', ...
				'fontsize', 9);
		end;
		

%-----------------------------------------------------------------------------
% UP:  handle mouseUp

	case 'UP',
		set(gcbf, ...						% clear motion handlers
			'WindowButtonMotionFcn', '', ...
			'WindowButtonUpFcn', '');
		
		label = struct('NAME', '', ...		% no name
					'OFFSET', [], ...		% use current cursor offset
					'NOTE', [], ...			% set by annotation dialog
					'VALUE', []);			% value contents ignored

		m_label('MAKE', label);				% create the label
		
	
%-----------------------------------------------------------------------------
% error

	otherwise,
		error(['ML_COLORS:  unrecognized action (', action, ')']);
	
end;

%=============================================================================
% SHOWBLURB

function ShowBlurb

figPos = get(0, 'ScreenSize');
width = 300;
height = 170;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];

cfg = dialog('Name', 'ML_COLORS', ...
	'Position', figPos, ...
	'ButtonDownFcn', '');

% about
blurb = ['Works like default MAVIS label handlers, except that the contents ' ...
			'of the label NOTE field are interpreted as the plotting color and ' ...
			'style of the label (e.g. r: will plot as a red dotted line).  An ' ...
			'RGB color triplet can also be used (e.g. [.5 .5 .5]).  Empty or ' ...
			'non-string NOTE fields produce a standard label.'];

uicontrol(cfg, ...
	'Position', [10 60 width-20 100], ...
	'Style', 'frame');
uicontrol(cfg, ...
	'Position', [13 63 width-26 94], ...
	'Style', 'text', ...
	'HorizontalAlignment', 'left', ...
	'String', blurb);

% OK button
uicontrol(cfg, ...		% buttons
	'Position',[width/2-30 15 60 25], ...
	'String','OK', ...
	'Callback','uiresume');

% wait for input
uiwait(cfg);
delete(cfg);
