function m_out = ml_extrema(action, aux, aux2, aux3)
%ML_EXTREMA  - MAVIS user labelling procedure
%
% Labels the minimum and maximum peaks of the selected dimension
% of the clicked trajectory

% 01/98 mkt

%	branch by action...

switch action,
		
%-----------------------------------------------------------------------------
% CONFIG:  handle configuration
%
%	AUX		- internal state
%
% which dimension should be tested?

	case 'CONFIG',
		state = aux;						% get internal state
		labelData = DoConfig(state.LABEL.USERDATA);
		if isempty(labelData),
			m_out = 0;				% report cancel
			return;
		end;
		state.LABEL.USERDATA = labelData;
		set(state.GUI, 'UserData', state);	% update internal state
		m_out = 1;							% report OK
		return;
		
		
%-----------------------------------------------------------------------------
% DOWN:  handle mouseDown
%
%	AUX		- internal state
%	AUX2	- index of clicked trajectory (into state.DATA)
%
% find the extrema of clicked trajectory
% if it has less than preferred dimension use highest that it has

	case 'DOWN',
		global msecsToSamps sampsToMsecs;	% map time : samples
		state = aux;						% internal state
		t = aux2;							% index of clicked trajectory
		d = state.LABEL.USERDATA.DIM;		% preferred dimension
		
% get clicked trajectory data over the range of the current temporal selection
		sRate = state.DATA(t).SRATE;
		head = msecsToSamps(state.HEAD, sRate);
		tail = msecsToSamps(state.TAIL, sRate);
		nDims = size(state.DATA(t).SIGNAL,2);
		if nDims < d, d = nDims; end;	% dimension
		data = state.DATA(t).SIGNAL(head:tail,d);

% find extrema
		[junk, minS] = min(data);					% samples
		minT = sampsToMsecs(minS+head-1, sRate);	% msecs
		[junk, maxS] = max(data);
		maxT = sampsToMsecs(maxS+head-1, sRate);

% label
		saveAnnotate = state.LABEL.ANNOTATE;		% get current annotation status
		state.LABEL.ANNOTATE = 0;					% force it off for quiet label creation
		set(state.GUI, 'UserData', state);

		trajName = state.DATA(t).NAME;				% build name 
		if nDims > 1,
			xyz = 'XYZ';
			trajName = [trajName '_' xyz(d)]; 
		end;
		label = struct('NAME', 'min', ...
					'OFFSET', minT, ...				
					'NOTE', [trajName ' (min)'], ...					
					'VALUE', []);
		m_label('MAKE', label);
		label = struct('NAME', 'max', ...
					'OFFSET', maxT, ...				
					'NOTE', [trajName ' (max)'], ...					
					'VALUE', []);
		m_label('MAKE', label);
		
		state = get(state.GUI, 'UserData');			% retrieve internal state
		state.LABEL.ANNOTATE = saveAnnotate;		% restore annotation status
		set(state.GUI, 'UserData', state);
		

%-----------------------------------------------------------------------------
% PLOT:  plot label into the current axes
%
%	AUX		- x offset (in trajectory sample units) of label
%	AUX2	- yLim of current axis
%	AUX3	- label
%
% call the default plotting routine

	case 'PLOT',
		m_label('PLOT', aux, aux2, aux3);
		

%-----------------------------------------------------------------------------
% error

	otherwise,
		error(['ML_EXTREMA:  unrecognized action (', action, ')']);
	
end;


%=============================================================================
% DOCONFIG  - config handler
%
%   returns non-empty labelData (selected dimension) on OK, empty result on cancel

function labelData = DoConfig(labelData)

figPos = get(0, 'ScreenSize');
width = 270;
height = 150;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];
if strncmp(computer, 'SGI', 3), as = 'left'; else, as = 'right'; end;	% SGI bug:  doesn't like right alignment

idString = 'ML_EXTREMA';
if isempty(labelData) | ~strcmp(labelData.SOURCE, idString),		% if not previously configured
	labelData = struct('SOURCE', idString, ...
						'DIM', 1);
end;

cfg = dialog('Name', idString, ...
	'Position', figPos, ...
	'ButtonDownFcn', '', ...
	'UserData', 0);

% about
uicontrol(cfg, ...
	'Position', [10 height-60 width-20 50], ...
	'Style', 'frame');
uicontrol(cfg, ...
	'Position', [13 height-57 width-26 44], ...
	'Style', 'text', ...
	'HorizontalAlignment', 'left', ...
	'String', ['Labels the minimum and maximum peaks of the selected dimension '...
				'of the clicked trajectory']);

% select dimensions
uicontrol(cfg, ...
	'Position', [10 55 115 17], ...
	'HorizontalAlignment', as, ...
	'String','Apply to dimension:', ...
	'Style','text');
xRB = uicontrol(cfg, ...
	'Position', [130 55 40 20], ...
	'String','X', ...
	'Style','radiobutton', ...
	'Value', (labelData.DIM==1), ...
	'Callback', 'set(gcbo,''Value'',1);set(get(gcbo,''UserData''),''Value'',~get(gcbo,''Value''));');
yRB = uicontrol(cfg, ...
	'Position', [175 55 40 20], ...
	'String','Y', ...
	'Style','radiobutton', ...
	'Value', (labelData.DIM==2), ...
	'Callback', 'set(gcbo,''Value'',1);set(get(gcbo,''UserData''),''Value'',~get(gcbo,''Value''));');
zRB = uicontrol(cfg, ...
	'Position', [220 55 40 20], ...
	'String','Z', ...
	'Style','radiobutton', ...
	'Value', (labelData.DIM==3), ...
	'Callback', 'set(gcbo,''Value'',1);set(get(gcbo,''UserData''),''Value'',~get(gcbo,''Value''));');
set(xRB, 'UserData', [yRB zRB]);
set(yRB, 'UserData', [xRB zRB]);
set(zRB, 'UserData', [xRB yRB]);

% OK, cancel buttons
uicontrol(cfg, ...		% buttons
	'Position',[width/2-70 15 60 25], ...
	'String','OK', ...
	'Callback','set(gcbf,''UserData'',1);uiresume');
uicontrol(cfg, ...
	'Position',[width/2+10 15 60 25], ...
	'String','Cancel', ...
	'Callback','uiresume');

% wait for input
uiwait(cfg);
if get(cfg, 'UserData'),
	labelData.DIM = find([get(xRB, 'Value') get(yRB, 'Value') get(zRB, 'Value')]);
else,
	labelData = [];
end;
delete(cfg);
