function m_out = ml_zoomer(action, aux, aux2, aux3)
%ML_ZOOMER  - MAVIS user labelling procedure
%
% Opens a new, zoomable window showing the contents of the clicked trajectory
% over the range of the current temporal selection (no labels are set).

% 01/98 mkt

%	branch by action...

switch action,
		
%-----------------------------------------------------------------------------
% CONFIG:  handle configuration
%
% no configuration necessary, just show blurb

	case 'CONFIG',
		ShowBlurb;
		m_out = 1;					% report OK
		return;
		
		
%-----------------------------------------------------------------------------
% DOWN:  handle mouseDown
%
%	AUX		- internal state
%	AUX2	- index of clicked trajectory (into state.DATA)
%
% create a new figure, plot the contents of the clicked trajectory, make it zoomable

	case 'DOWN',
		global msecsToSamps;	% map time to samples
		state = aux;			% internal state
		t = aux2;				% index of clicked trajectory
		
% get clicked trajectory data over the range of the current temporal selection
		head = msecsToSamps(state.HEAD, state.DATA(t).SRATE);
		tail = msecsToSamps(state.TAIL, state.DATA(t).SRATE);
		data = state.DATA(t).SIGNAL(head:tail,:);		% clicked trajectory

% means center
		[nSamps, nDims] = size(data);
		mins = state.DATA(t).SCALING.RANGE(:,1)';
		spreads = (state.DATA(t).SCALING.RANGE(:,2)' - mins) / 2;
		spread = max(spreads);
		spread = spread + spread * 0.2;		% squish slightly to fit within axis
		data = data(1:nSamps,:) - repmat(mins+spreads,nSamps,1);

% plot data
		figure('Name', state.DATA(t).NAME);
		plot(data);											% data
		set(gca, 'Xlim', [1 nSamps], 'Ylim', [-spread spread], 'Box', 'on');
		line('Xdata', [1 nSamps], 'Ydata', [0 0], ...
				'Color', [.5 .5 .5], 'LineStyle', ':');		% midline
		xlabel('samps');
		zoom on;
		

%-----------------------------------------------------------------------------
% PLOT:  plot label into the current axes
%
%	AUX		- x offset (in trajectory sample units) of label
%	AUX2	- yLim of current axis
%	AUX3	- label
%
% call the default plotting routine

	case 'PLOT',
		m_label('PLOT', aux, aux2, aux3);
		

%-----------------------------------------------------------------------------
% error

	otherwise,
		error(['ML_ZOOMER:  unrecognized action (', action, ')']);
	
end;


%=============================================================================
% SHOWBLURB

function ShowBlurb

figPos = get(0, 'ScreenSize');
width = 270;
height = 140;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];

cfg = dialog('Name', 'ML_EXTENTS', ...
	'Position', figPos, ...
	'ButtonDownFcn', '');

% about
blurb = ['Opens a new, zoomable window showing the contents of the clicked trajectory ' ...
		'over the range of the current temporal selection (no labels are set).'];


uicontrol(cfg, ...
	'Position', [10 60 width-20 70], ...
	'Style', 'frame');
uicontrol(cfg, ...
	'Position', [13 63 width-26 64], ...
	'Style', 'text', ...
	'HorizontalAlignment', 'left', ...
	'String', blurb);

% OK button
uicontrol(cfg, ...		% buttons
	'Position',[width/2-30 15 60 25], ...
	'String','OK', ...
	'Callback','uiresume');

% wait for input
uiwait(cfg);
delete(cfg);
