function h0 = menuas2f(menustr,callbackfn)
% MENUAS2F Creates a vertical menu
%		MENUAS2F creates a vertical menu of pushbuttons. The
%		first parameter defines the labels of the pushbuttons
%		and has to be a cell structure. The second parameter
%		is a string which specifies the callback routine when 
%		the buttons are pressed. This callback routine must
%		accept one parameter which is the index of the item.
%		The Tag of each menuitem is 'mi%d' where %d again
%		represents the index. The Tag of the menu is 'menuas2f.'

% Copyright (c) 1999 Satrajit S. Ghosh (satra@bu.edu)
% Department of Cognitive and Neural Systems, Boston University

% $Revision: 1.00$ $Date:Fri Oct  1 17:02:05 EDT 1999$

% Bug fixes

% Modifications

% Find any active menu and close it
obj = getmenuobj;
if (~(isempty(obj))),
   close(obj);
   disp('closing obj');
end;

% Get the data
obj1 = findobj('Tag','Vtcalcs');
data = get(obj1,'Userdata');

% Position menu to the right of existing figure
obj = findobj('Tag','Vtcalcs');
pos = get(obj,'position');

h0 = figure(...
   'units','normalized',...
   'menubar','none',...
   'Tag','menuas2f',...
   'Resize','off',...
   'numbertitle','off',...
   'position',[pos(1)+pos(3)+0.01 0.1 0.2 0.85]);
%   'Windowstyle','modal',...

% determine the height of each uicontrol object
wd = 0.95/(length(menustr)+data.AS2Fpar.nss);

% pushbuttons
for i=2:length(menustr),
   h1 = uicontrol(h0,...
      'units','normalized',...
      'position',[0.01 0.95-(i-1)*wd 0.98 0.8*wd],...
      'string',menustr(i),...
      'HorizontalAlignment','left',...
      'Busyaction','cancel',...
      'Tag',sprintf('mi%d',i),...
      'Userdata',menustr(i),...
      'callback',[callbackfn sprintf('(%d)',i)]);
   if (i==3),
      set(h1,'Enable','off');
   end;
end;

st = 0.95-(length(menustr))*wd;

% sliders
for i=1:data.AS2Fpar.nss,
   str = sprintf('[%2.2f]cm2',i,6.00);
   h1 = uicontrol(h0,...
      'units','normalized',...
      'position',[0.01 st-(i-1)*wd 0.3 0.8*wd],...
      'Style','text',...
      'String',str,...
      'Tag',sprintf('mistr%d',i));
   h1 = uicontrol(h0,...
      'units','normalized',...
      'position',[0.31 st-(i-1)*wd 0.68 0.8*wd],...
      'Style','slider',...
      'Busyaction','cancel',...
      'Min', 1,...
      'Max',1400,...
      'Value',1000,...
      'Tag',sprintf('misl%d',i),...
      'callback',[callbackfn sprintf('(%d)',i+length(menustr))]);
end;
